@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009,2010,2011 Red Hat, Inc.
+ * Copyright (C) 2009,2010,2011,2012,2013,2014 Red Hat, Inc.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -117,37 +117,51 @@ compare_items(const void *a, const void *b)
 	return SECITEM_CompareItem(a, b);
 }
 static SECItem *
-cm_csrgen_n_attributes(struct cm_store_entry *entry, PLArenaPool *arena)
+cm_csrgen_n_attributes(struct cm_store_entry *entry, NSSInitContext *ctx,
+		       PLArenaPool *arena)
 {
 	SECItem encoded_exts, *exts[2];
 	unsigned char *extensions;
+	char *nickname;
 	size_t extensions_length;
 	CERTAttribute attr[3];
 	SECOidData *oid;
 	SECItem *item, friendly, *friendlies[2], encoded, encattr[3], plain;
-	SECItem *encattrs[4], **encattrs_ptr, password, *passwords[2];
+	SECItem *encattrs[4], **encattrs_ptr, password, *passwords[2], bmp;
 	int i, n_attrs;
 
 	i = 0;
 	/* Build an attribute to hold the friendly name. */
 	oid = SECOID_FindOIDByTag(SEC_OID_PKCS9_FRIENDLY_NAME);
 	if (oid != NULL) {
-		plain.data = (unsigned char *) entry->cm_cert_nickname;
-		if (plain.data != NULL) {
-			plain.len = strlen(entry->cm_cert_nickname);
-			if (SEC_ASN1EncodeItem(arena, &friendly, &plain,
-					       SEC_PrintableStringTemplate) == &friendly) {
+		if (entry->cm_cert_nickname != NULL) {
+			nickname = entry->cm_cert_nickname;
+		} else
+		if (entry->cm_key_nickname != NULL) {
+			nickname = entry->cm_key_nickname;
+		} else {
+			nickname = entry->cm_nickname;
+		}
+		if (nickname != NULL) {
+			memset(&bmp, 0, sizeof(bmp));
+			if ((cm_store_utf8_to_bmp_string(nickname,
+							 &bmp.data,
+							 &bmp.len) == 0) &&
+			    (SEC_ASN1EncodeItem(arena, &friendly, &bmp,
+					        SEC_BMPStringTemplate) == &friendly)) {
 				friendlies[0] = &friendly;
 				friendlies[1] = NULL;
 				attr[i].attrType = oid->oid;
 				attr[i].attrValue = friendlies;
 				i++;
 			}
+			free(bmp.data);
 		}
 	}
 	/* Build the extension list. */
 	extensions = NULL;
-	cm_certext_build_csr_extensions(entry, &extensions, &extensions_length);
+	cm_certext_build_csr_extensions(entry, ctx, &extensions,
+					&extensions_length);
 	/* Build an attribute to hold the extensions. */
 	if ((extensions != NULL) && (extensions_length > 0)) {
 		encoded_exts.data = extensions;
@@ -164,6 +178,7 @@ cm_csrgen_n_attributes(struct cm_store_entry *entry, PLArenaPool *arena)
 	/* Build an attribute to hold the challenge password. */
 	oid = SECOID_FindOIDByTag(SEC_OID_PKCS9_CHALLENGE_PASSWORD);
 	if (oid != NULL) {
+		memset(&plain, 0, sizeof(plain));
 		plain.data = (unsigned char *) entry->cm_challenge_password;
 		if (plain.data != NULL) {
 			plain.len = strlen(entry->cm_challenge_password);
@@ -219,112 +234,181 @@ cm_csrgen_n_main(int fd, struct cm_store_ca *ca, struct cm_store_entry *entry,
 {
 	FILE *status;
 	SECStatus error;
-	struct cm_keyiread_n_ctx_and_key *privkey;
-	SECKEYPublicKey *pubkey;
+	struct cm_keyiread_n_ctx_and_keys *keys;
 	CERTSubjectPublicKeyInfo *spki;
 	CERTPublicKeyAndChallenge pkac;
 	CERTCertificateRequest *req;
 	CERTSignedData sreq, spkac;
 	CERTName *name;
 	PLArenaPool *arena;
-	SECItem ereq, esreq, epkac, espkac, *attrs;
-	PRErrorCode ec;
+	SECItem ereq, esreq, epkac, espkac, *attrs, item, utf8;
+	int ec;
 	char *b64, *b642, *p, *q;
+	const char *es;
 	SECOidData *sigoid;
 
 	/* Allocate an arena pool and a place to write status updates. */
 	arena = PORT_NewArena(sizeof(double));
 	if (arena == NULL) {
 		cm_log(1, "Out of memory?.\n");
-		_exit(CM_STATUS_ERROR_INTERNAL);
+		_exit(CM_SUB_STATUS_INTERNAL_ERROR);
 	}
 	status = fdopen(fd, "w");
 	if (status == NULL) {
 		cm_log(1, "Internal error: %s.\n", strerror(errno));
-		_exit(CM_STATUS_ERROR_INTERNAL);
+		_exit(CM_SUB_STATUS_INTERNAL_ERROR);
 	}
 
 	/* Start up NSS and find the key pair. */
-	privkey = cm_keyiread_n_get_private_key(entry, 0);
-	if (privkey == NULL) {
+	keys = cm_keyiread_n_get_keys(entry, 0);
+	if (keys == NULL) {
 		cm_log(1, "Error finding key pair for %s('%s').\n",
 		       entry->cm_busname, entry->cm_nickname);
 		PORT_FreeArena(arena, PR_TRUE);
-		_exit(CM_STATUS_ERROR_NO_TOKEN);
+		_exit(CM_SUB_STATUS_ERROR_NO_TOKEN);
 	}
 	/* Select a subject name. */
-	if ((entry->cm_template_subject != NULL) &&
+	name = NULL;
+	if ((entry->cm_template_subject_der != NULL) &&
+	    (strlen(entry->cm_template_subject_der) != 0)) {
+		memset(&item, 0, sizeof(item));
+		item.len = strlen(entry->cm_template_subject_der) / 2;
+		item.data = malloc(item.len);
+		if (item.data != NULL) {
+			item.len = cm_store_hex_to_bin(entry->cm_template_subject_der,
+						       item.data, item.len);
+			name = PORT_ArenaZNew(arena, CERTName);
+			if (name != NULL) {
+				if (SEC_ASN1DecodeItem(arena, name,
+						       CERT_NameTemplate,
+						       &item) != SECSuccess) {
+					name = NULL;
+				}
+			}
+		}
+		if (name == NULL) {
+			cm_log(1, "Error parsing requested subject \"%s\".\n",
+			       entry->cm_template_subject_der);
+		}
+	}
+	if ((name == NULL) &&
+	    (entry->cm_template_subject != NULL) &&
 	    (strlen(entry->cm_template_subject) != 0)) {
 		name = CERT_AsciiToName(entry->cm_template_subject);
-	} else {
-		name = CERT_AsciiToName("CN=" CM_DEFAULT_CERT_SUBJECT_CN);
-	}
-	/* Find the public key. */
-	pubkey = SECKEY_ConvertToPublicKey(privkey->key);
-	if (pubkey == NULL) {
-		ec = PR_GetError();
-		if (ec == 0) {
-			cm_log(1, "Error retrieving public key.\n");
-		} else {
-			cm_log(1, "Error retrieving public key: %s.\n",
-			       PR_ErrorToString(ec, PR_LANGUAGE_I_DEFAULT));
+		if (name == NULL) {
+			/* Force it. */
+			memset(&item, 0, sizeof(item));
+			item.data = (unsigned char *) entry->cm_template_subject;
+			item.len = strlen(entry->cm_template_subject);
+			memset(&utf8, 0, sizeof(utf8));
+			if (SEC_ASN1EncodeItem(arena, &utf8, &item,
+					       SEC_PrintableStringTemplate) == &utf8) {
+				q = cm_store_hex_from_bin(NULL,
+							  utf8.data,
+							  utf8.len);
+				if (q != NULL) {
+					p = talloc_asprintf(q, "CN=#%s", q);
+					if (p != NULL) {
+						name = CERT_AsciiToName(p);
+					}
+					talloc_free(q);
+				}
+			}
 		}
-		SECKEY_DestroyPublicKey(pubkey);
-		SECKEY_DestroyPrivateKey(privkey->key);
+		if (name == NULL) {
+			cm_log(1, "Error parsing requested subject name \"%s\".\n",
+			       entry->cm_template_subject);
+		}
+	}
+	if (name == NULL) {
+		name = CERT_AsciiToName("CN=" CM_DEFAULT_CERT_SUBJECT_CN);
+		if (name == NULL) {
+			cm_log(1, "Error parsing requested subject name \"%s\".\n",
+			       "CN=" CM_DEFAULT_CERT_SUBJECT_CN);
+		}
+	}
+	if (name == NULL) {
+		if (keys->pubkey != NULL) {
+			SECKEY_DestroyPublicKey(keys->pubkey);
+		}
+		SECKEY_DestroyPrivateKey(keys->privkey);
 		PORT_FreeArena(arena, PR_TRUE);
-		error = NSS_ShutdownContext(privkey->ctx);
-		PORT_FreeArena(privkey->arena, PR_TRUE);
+		error = NSS_ShutdownContext(keys->ctx);
+		PORT_FreeArena(keys->arena, PR_TRUE);
 		if (error != SECSuccess) {
 			cm_log(1, "Error shutting down NSS.\n");
 		}
 		fclose(status);
-		_exit(CM_STATUS_ERROR_INTERNAL);
+		_exit(CM_SUB_STATUS_INTERNAL_ERROR);
+	}
+	/* Find the public key. */
+	if (keys->pubkey == NULL) {
+		ec = PORT_GetError();
+		if (ec != 0) {
+			es = PR_ErrorToName(ec);
+		} else {
+			es = NULL;
+		}
+		if (es != NULL) {
+			cm_log(1, "Error retrieving public key: %s.\n", es);
+		} else {
+			cm_log(1, "Error retrieving public key: %d.\n", ec);
+		}
+		SECKEY_DestroyPrivateKey(keys->privkey);
+		PORT_FreeArena(arena, PR_TRUE);
+		error = NSS_ShutdownContext(keys->ctx);
+		PORT_FreeArena(keys->arena, PR_TRUE);
+		if (error != SECSuccess) {
+			cm_log(1, "Error shutting down NSS.\n");
+		}
+		fclose(status);
+		_exit(CM_SUB_STATUS_INTERNAL_ERROR);
 	}
 	/* Generate a subjectPublicKeyInfo. */
-	spki = SECKEY_CreateSubjectPublicKeyInfo(pubkey);
+	spki = SECKEY_CreateSubjectPublicKeyInfo(keys->pubkey);
 	if (spki == NULL) {
-		ec = PR_GetError();
+		ec = PORT_GetError();
 		if (ec == 0) {
 			cm_log(1, "Error building spki value.\n");
 		} else {
 			cm_log(1, "Error building spki value: %s.\n",
-			       PR_ErrorToString(ec, PR_LANGUAGE_I_DEFAULT));
+			       PR_ErrorToName(ec));
 		}
-		SECKEY_DestroyPublicKey(pubkey);
-		SECKEY_DestroyPrivateKey(privkey->key);
+		SECKEY_DestroyPublicKey(keys->pubkey);
+		SECKEY_DestroyPrivateKey(keys->privkey);
 		PORT_FreeArena(arena, PR_TRUE);
-		error = NSS_ShutdownContext(privkey->ctx);
-		PORT_FreeArena(privkey->arena, PR_TRUE);
+		error = NSS_ShutdownContext(keys->ctx);
+		PORT_FreeArena(keys->arena, PR_TRUE);
 		if (error != SECSuccess) {
 			cm_log(1, "Error shutting down NSS.\n");
 		}
 		fclose(status);
-		_exit(CM_STATUS_ERROR_INTERNAL);
+		_exit(CM_SUB_STATUS_INTERNAL_ERROR);
 	}
 	/* Build the request. */
 	req = CERT_CreateCertificateRequest(name, spki, NULL);
 	if (req == NULL) {
-		ec = PR_GetError();
+		ec = PORT_GetError();
 		if (ec == 0) {
 			cm_log(1, "Error building certificate request.\n");
 		} else {
 			cm_log(1, "Error building certificate request: %s.\n",
-			       PR_ErrorToString(ec, PR_LANGUAGE_I_DEFAULT));
+			       PR_ErrorToName(ec));
 		}
-		SECKEY_DestroyPublicKey(pubkey);
-		SECKEY_DestroyPrivateKey(privkey->key);
+		SECKEY_DestroyPublicKey(keys->pubkey);
+		SECKEY_DestroyPrivateKey(keys->privkey);
 		PORT_FreeArena(arena, PR_TRUE);
-		error = NSS_ShutdownContext(privkey->ctx);
-		PORT_FreeArena(privkey->arena, PR_TRUE);
+		error = NSS_ShutdownContext(keys->ctx);
+		PORT_FreeArena(keys->arena, PR_TRUE);
 		if (error != SECSuccess) {
 			cm_log(1, "Error shutting down NSS.\n");
 		}
 		fclose(status);
-		_exit(CM_STATUS_ERROR_INTERNAL);
+		_exit(CM_SUB_STATUS_INTERNAL_ERROR);
 	}
 	/* Generate requested values for various extensions and a friendly
 	 * name. */
-	attrs = cm_csrgen_n_attributes(entry, arena);
+	attrs = cm_csrgen_n_attributes(entry, keys->ctx, arena);
 	if ((attrs == NULL) ||
 	    (SEC_ASN1DecodeItem(arena, &req->attributes,
 				cm_csrgen_n_set_of_cert_tmpattr_template,
@@ -343,16 +427,16 @@ cm_csrgen_n_main(int fd, struct cm_store_ca *ca, struct cm_store_entry *entry,
 			       CERT_CertificateRequestTemplate) !=
 	    &ereq) {
 		cm_log(1, "Error encoding certificate request.\n");
-		SECKEY_DestroyPublicKey(pubkey);
-		SECKEY_DestroyPrivateKey(privkey->key);
+		SECKEY_DestroyPublicKey(keys->pubkey);
+		SECKEY_DestroyPrivateKey(keys->privkey);
 		PORT_FreeArena(arena, PR_TRUE);
-		error = NSS_ShutdownContext(privkey->ctx);
-		PORT_FreeArena(privkey->arena, PR_TRUE);
+		error = NSS_ShutdownContext(keys->ctx);
+		PORT_FreeArena(keys->arena, PR_TRUE);
 		if (error != SECSuccess) {
 			cm_log(1, "Error shutting down NSS.\n");
 		}
 		fclose(status);
-		_exit(CM_STATUS_ERROR_INTERNAL);
+		_exit(CM_SUB_STATUS_INTERNAL_ERROR);
 	}
 	/* Build the PublicKeyAndChallenge. */
 	memset(&pkac, 0, sizeof(pkac));
@@ -360,16 +444,16 @@ cm_csrgen_n_main(int fd, struct cm_store_ca *ca, struct cm_store_entry *entry,
 			       CERT_SubjectPublicKeyInfoTemplate) !=
 	    &pkac.spki) {
 		cm_log(1, "Error encoding subject public key info.\n");
-		SECKEY_DestroyPublicKey(pubkey);
-		SECKEY_DestroyPrivateKey(privkey->key);
+		SECKEY_DestroyPublicKey(keys->pubkey);
+		SECKEY_DestroyPrivateKey(keys->privkey);
 		PORT_FreeArena(arena, PR_TRUE);
-		error = NSS_ShutdownContext(privkey->ctx);
-		PORT_FreeArena(privkey->arena, PR_TRUE);
+		error = NSS_ShutdownContext(keys->ctx);
+		PORT_FreeArena(keys->arena, PR_TRUE);
 		if (error != SECSuccess) {
 			cm_log(1, "Error shutting down NSS.\n");
 		}
 		fclose(status);
-		_exit(CM_STATUS_ERROR_INTERNAL);
+		_exit(CM_SUB_STATUS_INTERNAL_ERROR);
 	}
 	pkac.challenge.data = (unsigned char *) entry->cm_challenge_password;
 	pkac.challenge.len = entry->cm_challenge_password ?
@@ -379,50 +463,51 @@ cm_csrgen_n_main(int fd, struct cm_store_ca *ca, struct cm_store_entry *entry,
 			       cm_csrgen_n_cert_pkac_template) !=
 	    &epkac) {
 		cm_log(1, "Error encoding public key and challenge.\n");
-		SECKEY_DestroyPublicKey(pubkey);
-		SECKEY_DestroyPrivateKey(privkey->key);
+		SECKEY_DestroyPublicKey(keys->pubkey);
+		SECKEY_DestroyPrivateKey(keys->privkey);
 		PORT_FreeArena(arena, PR_TRUE);
-		error = NSS_ShutdownContext(privkey->ctx);
-		PORT_FreeArena(privkey->arena, PR_TRUE);
+		error = NSS_ShutdownContext(keys->ctx);
+		PORT_FreeArena(keys->arena, PR_TRUE);
 		if (error != SECSuccess) {
 			cm_log(1, "Error shutting down NSS.\n");
 		}
 		fclose(status);
-		_exit(CM_STATUS_ERROR_INTERNAL);
+		_exit(CM_SUB_STATUS_INTERNAL_ERROR);
 	}
 	/* Sign the request using the private key. */
-	sigoid = SECOID_FindOIDByTag(cm_prefs_nss_sig_alg(pubkey));
+	sigoid = SECOID_FindOIDByTag(cm_prefs_nss_sig_alg(keys->privkey));
 	memset(&sreq, 0, sizeof(sreq));
 	sreq.data = ereq;
 	if (SECOID_SetAlgorithmID(arena, &sreq.signatureAlgorithm,
 				  sigoid->offset, NULL) != SECSuccess) {
 		cm_log(1, "Error setting up algorithm ID for signing the "
 		       "certificate request.\n");
-		SECKEY_DestroyPublicKey(pubkey);
-		SECKEY_DestroyPrivateKey(privkey->key);
+		SECKEY_DestroyPublicKey(keys->pubkey);
+		SECKEY_DestroyPrivateKey(keys->privkey);
 		PORT_FreeArena(arena, PR_TRUE);
-		error = NSS_ShutdownContext(privkey->ctx);
-		PORT_FreeArena(privkey->arena, PR_TRUE);
+		error = NSS_ShutdownContext(keys->ctx);
+		PORT_FreeArena(keys->arena, PR_TRUE);
 		if (error != SECSuccess) {
 			cm_log(1, "Error shutting down NSS.\n");
 		}
 		fclose(status);
-		_exit(CM_STATUS_ERROR_INTERNAL);
+		_exit(CM_SUB_STATUS_INTERNAL_ERROR);
 	}
 	if (SEC_SignData(&sreq.signature, sreq.data.data, sreq.data.len,
-			 privkey->key, sigoid->offset) != SECSuccess) {
+			 keys->privkey, sigoid->offset) != SECSuccess) {
 		cm_log(1, "Error signing certificate request with the client's "
-		       "key.\n");
-		SECKEY_DestroyPublicKey(pubkey);
-		SECKEY_DestroyPrivateKey(privkey->key);
+		       "key using \"%s\": %s.\n",
+		       sigoid->desc, PR_ErrorToName(PORT_GetError()));
+		SECKEY_DestroyPublicKey(keys->pubkey);
+		SECKEY_DestroyPrivateKey(keys->privkey);
 		PORT_FreeArena(arena, PR_TRUE);
-		error = NSS_ShutdownContext(privkey->ctx);
-		PORT_FreeArena(privkey->arena, PR_TRUE);
+		error = NSS_ShutdownContext(keys->ctx);
+		PORT_FreeArena(keys->arena, PR_TRUE);
 		if (error != SECSuccess) {
 			cm_log(1, "Error shutting down NSS.\n");
 		}
 		fclose(status);
-		_exit(CM_STATUS_ERROR_INTERNAL);
+		_exit(CM_SUB_STATUS_INTERNAL_ERROR);
 	}
 	/* Sign the PublicKeyAndChallenge using the private key. */
 	memset(&spkac, 0, sizeof(spkac));
@@ -431,31 +516,32 @@ cm_csrgen_n_main(int fd, struct cm_store_ca *ca, struct cm_store_entry *entry,
 				  sigoid->offset, NULL) != SECSuccess) {
 		cm_log(1, "Error setting up algorithm ID for signing the "
 		       "certificate request.\n");
-		SECKEY_DestroyPublicKey(pubkey);
-		SECKEY_DestroyPrivateKey(privkey->key);
+		SECKEY_DestroyPublicKey(keys->pubkey);
+		SECKEY_DestroyPrivateKey(keys->privkey);
 		PORT_FreeArena(arena, PR_TRUE);
-		error = NSS_ShutdownContext(privkey->ctx);
-		PORT_FreeArena(privkey->arena, PR_TRUE);
+		error = NSS_ShutdownContext(keys->ctx);
+		PORT_FreeArena(keys->arena, PR_TRUE);
 		if (error != SECSuccess) {
 			cm_log(1, "Error shutting down NSS.\n");
 		}
 		fclose(status);
-		_exit(CM_STATUS_ERROR_INTERNAL);
+		_exit(CM_SUB_STATUS_INTERNAL_ERROR);
 	}
 	if (SEC_SignData(&spkac.signature, spkac.data.data, spkac.data.len,
-			 privkey->key, sigoid->offset) != SECSuccess) {
+			 keys->privkey, sigoid->offset) != SECSuccess) {
 		cm_log(1, "Error signing public-key-and-challenge with "
-		       "the client's key.\n");
-		SECKEY_DestroyPublicKey(pubkey);
-		SECKEY_DestroyPrivateKey(privkey->key);
+		       "the client's key using \"%s\": %s.\n",
+		       sigoid->desc, PR_ErrorToName(PORT_GetError()));
+		SECKEY_DestroyPublicKey(keys->pubkey);
+		SECKEY_DestroyPrivateKey(keys->privkey);
 		PORT_FreeArena(arena, PR_TRUE);
-		error = NSS_ShutdownContext(privkey->ctx);
-		PORT_FreeArena(privkey->arena, PR_TRUE);
+		error = NSS_ShutdownContext(keys->ctx);
+		PORT_FreeArena(keys->arena, PR_TRUE);
 		if (error != SECSuccess) {
 			cm_log(1, "Error shutting down NSS.\n");
 		}
 		fclose(status);
-		_exit(CM_STATUS_ERROR_INTERNAL);
+		_exit(CM_SUB_STATUS_INTERNAL_ERROR);
 	}
 	/* Encode the signed request. */
 	sreq.signature.len *= 8;
@@ -463,16 +549,16 @@ cm_csrgen_n_main(int fd, struct cm_store_ca *ca, struct cm_store_entry *entry,
 			       CERT_SignedDataTemplate) !=
 	    &esreq) {
 		cm_log(1, "Error encoding signed certificate request.\n");
-		SECKEY_DestroyPublicKey(pubkey);
-		SECKEY_DestroyPrivateKey(privkey->key);
+		SECKEY_DestroyPublicKey(keys->pubkey);
+		SECKEY_DestroyPrivateKey(keys->privkey);
 		PORT_FreeArena(arena, PR_TRUE);
-		error = NSS_ShutdownContext(privkey->ctx);
-		PORT_FreeArena(privkey->arena, PR_TRUE);
+		error = NSS_ShutdownContext(keys->ctx);
+		PORT_FreeArena(keys->arena, PR_TRUE);
 		if (error != SECSuccess) {
 			cm_log(1, "Error shutting down NSS.\n");
 		}
 		fclose(status);
-		_exit(CM_STATUS_ERROR_INTERNAL);
+		_exit(CM_SUB_STATUS_INTERNAL_ERROR);
 	}
 	/* Encode the signed public key and challenge. */
 	spkac.signature.len *= 8;
@@ -480,16 +566,16 @@ cm_csrgen_n_main(int fd, struct cm_store_ca *ca, struct cm_store_entry *entry,
 			       CERT_SignedDataTemplate) !=
 	    &espkac) {
 		cm_log(1, "Error encoding signed public key and challenge.\n");
-		SECKEY_DestroyPublicKey(pubkey);
-		SECKEY_DestroyPrivateKey(privkey->key);
+		SECKEY_DestroyPublicKey(keys->pubkey);
+		SECKEY_DestroyPrivateKey(keys->privkey);
 		PORT_FreeArena(arena, PR_TRUE);
-		error = NSS_ShutdownContext(privkey->ctx);
-		PORT_FreeArena(privkey->arena, PR_TRUE);
+		error = NSS_ShutdownContext(keys->ctx);
+		PORT_FreeArena(keys->arena, PR_TRUE);
 		if (error != SECSuccess) {
 			cm_log(1, "Error shutting down NSS.\n");
 		}
 		fclose(status);
-		_exit(CM_STATUS_ERROR_INTERNAL);
+		_exit(CM_SUB_STATUS_INTERNAL_ERROR);
 	}
 	/* Encode the request into base-64 and pass it to our caller. */
 	b64 = NSSBase64_EncodeItem(arena, NULL, -1, &esreq);
@@ -509,11 +595,11 @@ cm_csrgen_n_main(int fd, struct cm_store_ca *ca, struct cm_store_entry *entry,
 			fprintf(status, "%.*s", (int) (q - p), p);
 			p = q + strspn(q, "\r\n");
 		}
-		SECKEY_DestroyPublicKey(pubkey);
-		SECKEY_DestroyPrivateKey(privkey->key);
+		SECKEY_DestroyPublicKey(keys->pubkey);
+		SECKEY_DestroyPrivateKey(keys->privkey);
 		PORT_FreeArena(arena, PR_TRUE);
-		error = NSS_ShutdownContext(privkey->ctx);
-		PORT_FreeArena(privkey->arena, PR_TRUE);
+		error = NSS_ShutdownContext(keys->ctx);
+		PORT_FreeArena(keys->arena, PR_TRUE);
 		if (error != SECSuccess) {
 			cm_log(1, "Error shutting down NSS.\n");
 		}
@@ -521,16 +607,16 @@ cm_csrgen_n_main(int fd, struct cm_store_ca *ca, struct cm_store_entry *entry,
 		_exit(0);
 	}
 	/* Clean up. */
-	SECKEY_DestroyPublicKey(pubkey);
-	SECKEY_DestroyPrivateKey(privkey->key);
+	SECKEY_DestroyPublicKey(keys->pubkey);
+	SECKEY_DestroyPrivateKey(keys->privkey);
 	PORT_FreeArena(arena, PR_TRUE);
-	error = NSS_ShutdownContext(privkey->ctx);
-	PORT_FreeArena(privkey->arena, PR_TRUE);
+	error = NSS_ShutdownContext(keys->ctx);
+	PORT_FreeArena(keys->arena, PR_TRUE);
 	if (error != SECSuccess) {
 		cm_log(1, "Error shutting down NSS.\n");
 	}
 	fclose(status);
-	_exit(CM_STATUS_ERROR_INTERNAL);
+	_exit(CM_SUB_STATUS_INTERNAL_ERROR);
 }
 
 /* Check if a CSR is ready. */
@@ -590,7 +676,7 @@ cm_csrgen_n_need_pin(struct cm_store_entry *entry,
 	int status;
 	status = cm_subproc_get_exitstatus(entry, state->subproc);
 	if (WIFEXITED(status) &&
-	    (WEXITSTATUS(status) == CM_STATUS_ERROR_AUTH)) {
+	    (WEXITSTATUS(status) == CM_SUB_STATUS_ERROR_AUTH)) {
 		return 0;
 	}
 	return -1;
@@ -604,7 +690,7 @@ cm_csrgen_n_need_token(struct cm_store_entry *entry,
 	int status;
 	status = cm_subproc_get_exitstatus(entry, state->subproc);
 	if (WIFEXITED(status) &&
-	    (WEXITSTATUS(status) == CM_STATUS_ERROR_NO_TOKEN)) {
+	    (WEXITSTATUS(status) == CM_SUB_STATUS_ERROR_NO_TOKEN)) {
 		return 0;
 	}
 	return -1;
