@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009,2010,2011 Red Hat, Inc.
+ * Copyright (C) 2009,2010,2011,2012,2013,2014 Red Hat, Inc.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -57,49 +57,51 @@ cm_csrgen_o_main(int fd, struct cm_store_ca *ca, struct cm_store_entry *entry,
 {
 	struct cm_pin_cb_data cb_data;
 	FILE *keyfp, *status;
-	X509 *x;
 	X509_REQ *req;
+	X509_NAME *subject;
 	NETSCAPE_SPKI spki;
 	NETSCAPE_SPKAC spkac;
-	RSA *rsa;
 	EVP_PKEY *pkey;
 	char buf[LINE_MAX], *p, *q, *s, *nickname, *pin, *password;
-	unsigned char *extensions, *unickname, *upassword;
+	unsigned char *extensions, *upassword, *bmp, *name;
 	const char *default_cn = CM_DEFAULT_CERT_SUBJECT_CN;
+	const unsigned char *nametmp;
 	size_t extensions_len;
+	unsigned int bmpcount;
 	long error;
 	int i;
 
 	status = fdopen(fd, "w");
 	if (status == NULL) {
-		_exit(CM_STATUS_ERROR_INTERNAL);
+		_exit(CM_SUB_STATUS_INTERNAL_ERROR);
 	}
 	keyfp = fopen(entry->cm_key_storage_location, "r");
 	if (keyfp == NULL) {
 		if (errno != ENOENT) {
 			cm_log(1, "Error opening key file \"%s\" "
-			       "for reading.\n",
-			       entry->cm_key_storage_location);
+			       "for reading: %s.\n",
+			       entry->cm_key_storage_location,
+			       strerror(errno));
 		}
-		_exit(CM_STATUS_ERROR_INTERNAL);
+		_exit(CM_SUB_STATUS_INTERNAL_ERROR);
 	}
 	util_o_init();
 	ERR_load_crypto_strings();
 	pkey = EVP_PKEY_new();
 	if (pkey == NULL) {
 		cm_log(1, "Internal error generating CSR.\n");
-		_exit(CM_STATUS_ERROR_INTERNAL);
+		_exit(CM_SUB_STATUS_INTERNAL_ERROR);
 	}
 	if (cm_pin_read_for_key(entry, &pin) != 0) {
 		cm_log(1, "Internal error reading key encryption PIN.\n");
-		_exit(CM_STATUS_ERROR_AUTH);
+		_exit(CM_SUB_STATUS_ERROR_AUTH);
 	}
 	memset(&cb_data, 0, sizeof(cb_data));
 	cb_data.entry = entry;
 	cb_data.n_attempts = 0;
-	rsa = PEM_read_RSAPrivateKey(keyfp, NULL,
-				     cm_pin_read_for_key_ossl_cb, &cb_data);
-	if (rsa == NULL) {
+	pkey = PEM_read_PrivateKey(keyfp, NULL,
+				   cm_pin_read_for_key_ossl_cb, &cb_data);
+	if (pkey == NULL) {
 		error = errno;
 		cm_log(1, "Error reading private key '%s': %s.\n",
 		       entry->cm_key_storage_location, strerror(error));
@@ -107,7 +109,7 @@ cm_csrgen_o_main(int fd, struct cm_store_ca *ca, struct cm_store_entry *entry,
 			ERR_error_string_n(error, buf, sizeof(buf));
 			cm_log(1, "%s\n", buf);
 		}
-		_exit(CM_STATUS_ERROR_AUTH); /* XXX */
+		_exit(CM_SUB_STATUS_ERROR_AUTH); /* XXX */
 	} else {
 		if ((pin != NULL) &&
 		    (strlen(pin) > 0) &&
@@ -120,116 +122,131 @@ cm_csrgen_o_main(int fd, struct cm_store_ca *ca, struct cm_store_entry *entry,
 				ERR_error_string_n(error, buf, sizeof(buf));
 				cm_log(1, "%s\n", buf);
 			}
-			_exit(CM_STATUS_ERROR_AUTH); /* XXX */
+			_exit(CM_SUB_STATUS_ERROR_AUTH); /* XXX */
 		}
 	}
-	if (rsa != NULL) {
-		EVP_PKEY_assign_RSA(pkey, rsa); /* pkey owns rsa now */
-		x = X509_new();
-		if (x != NULL) {
-			if (entry->cm_template_subject != NULL) {
+	if (pkey != NULL) {
+		req = X509_REQ_new();
+		if (req != NULL) {
+			subject = NULL;
+			if ((entry->cm_template_subject_der != NULL) &&
+			    (strlen(entry->cm_template_subject_der) != 0)) {
+				i = strlen(entry->cm_template_subject_der);
+				name = malloc(i);
+				if (name != NULL) {
+					i = cm_store_hex_to_bin(entry->cm_template_subject_der,
+								name, i);
+					nametmp = name;
+					subject = d2i_X509_NAME(NULL, &nametmp, i);
+				}
+			}
+			if ((subject == NULL) &&
+			    (entry->cm_template_subject != NULL) &&
+			    (strlen(entry->cm_template_subject) != 0)) {
 				/* This isn't really correct, but it will
 				 * probably do for now. */
 				p = entry->cm_template_subject;
 				q = p + strcspn(p, ",");
-				while (*p != '\0') {
-					if ((s = memchr(p, '=', q - p)) != NULL) {
-						*s = '\0';
-						for (i = 0; p[i] != '\0'; i++) {
-							p[i] = toupper(p[i]);
+				subject = X509_NAME_new();
+				if (subject != NULL) {
+					while (*p != '\0') {
+						if ((s = memchr(p, '=', q - p)) != NULL) {
+							*s = '\0';
+							for (i = 0; p[i] != '\0'; i++) {
+								p[i] = toupper(p[i]);
+							}
+							X509_NAME_add_entry_by_txt(subject,
+										   p, MBSTRING_UTF8,
+										   (unsigned char *) (s + 1), q - s - 1,
+										   -1, 0);
+							*s = '=';
+						} else {
+							X509_NAME_add_entry_by_txt(subject,
+										   "CN", MBSTRING_UTF8,
+										   (unsigned char *) p, q - p,
+										   -1, 0);
 						}
-						X509_NAME_add_entry_by_txt(x->cert_info->subject,
-									   p, MBSTRING_UTF8,
-									   (unsigned char *) (s + 1), q - s - 1,
-									   -1, 0);
-						*s = '=';
-					} else {
-						X509_NAME_add_entry_by_txt(x->cert_info->subject,
-									   "CN", MBSTRING_UTF8,
-									   (unsigned char *) p, q - p,
-									   -1, 0);
+						p = q + strspn(q, ",");
+						q = p + strcspn(p, ",");
 					}
-					p = q + strspn(q, ",");
-					q = p + strcspn(p, ",");
 				}
-			} else {
-				X509_NAME_add_entry_by_txt(x->cert_info->subject,
-							   "CN", MBSTRING_UTF8,
-							   (const unsigned char *) default_cn,
-							   strlen(default_cn),
-							   -1, 0);
 			}
-			X509_set_pubkey(x, pkey);
-			req = X509_to_X509_REQ(x, pkey, cm_prefs_ossl_hash());
-			if (req != NULL) {
-				/* Add attributes. */
-				extensions = NULL;
-				cm_certext_build_csr_extensions(entry,
-								&extensions,
-								&extensions_len);
-				if ((extensions != NULL) &&
-				    (extensions_len> 0)) {
-					X509_REQ_add1_attr_by_NID(req,
-								  NID_ext_req,
-								  V_ASN1_SEQUENCE,
-								  extensions,
-								  extensions_len);
-					talloc_free(extensions);
-				}
-				if (entry->cm_cert_nickname != NULL) {
-					nickname = entry->cm_cert_nickname;
-				} else {
-					nickname = entry->cm_nickname;
-				}
-				unickname = (unsigned char *) nickname;
-				if (nickname != NULL) {
-					X509_REQ_add1_attr_by_NID(req,
-								  NID_friendlyName,
-								  V_ASN1_PRINTABLESTRING,
-								  unickname,
-								  strlen(nickname));
-				}
-				password = entry->cm_challenge_password;
-				upassword = (unsigned char *) password;
-				if (password != NULL) {
-					X509_REQ_add1_attr_by_NID(req,
-								  NID_pkcs9_challengePassword,
-								  V_ASN1_PRINTABLESTRING,
-								  upassword,
-								  strlen(password));
-				}
-				X509_REQ_sign(req, pkey, cm_prefs_ossl_hash());
-				PEM_write_X509_REQ_NEW(status, req);
-				memset(&spkac, 0, sizeof(spkac));
-				spkac.challenge = M_ASN1_IA5STRING_new();
-				if (entry->cm_challenge_password != NULL) {
-					ASN1_STRING_set(spkac.challenge,
-							entry->cm_challenge_password,
-							strlen(entry->cm_challenge_password));
-				} else {
-					ASN1_STRING_set(spkac.challenge,
-							"", 0);
-				}
-				memset(&spki, 0, sizeof(spki));
-				spki.spkac = &spkac;
-				spki.sig_algor = req->sig_alg;
-				spki.signature = M_ASN1_BIT_STRING_new();
-				NETSCAPE_SPKI_set_pubkey(&spki, pkey);
-				NETSCAPE_SPKI_sign(&spki, pkey, cm_prefs_ossl_hash());
-				s = NETSCAPE_SPKI_b64_encode(&spki);
-				if (s != NULL) {
-					fprintf(status, "%s\n", s);
+			if (subject == NULL) {
+				subject = X509_NAME_new();
+				if (subject != NULL) {
+					X509_NAME_add_entry_by_txt(subject,
+								   "CN", MBSTRING_UTF8,
+								   (const unsigned char *) default_cn,
+								   -1, -1, 0);
 				}
+			}
+			if (subject != NULL) {
+				X509_NAME_set(&req->req_info->subject, subject);
+			}
+			X509_REQ_set_pubkey(req, pkey);
+			X509_REQ_set_version(req, SEC_CERTIFICATE_REQUEST_VERSION);
+			/* Add attributes. */
+			extensions = NULL;
+			cm_certext_build_csr_extensions(entry, NULL,
+							&extensions,
+							&extensions_len);
+			if ((extensions != NULL) &&
+			    (extensions_len> 0)) {
+				X509_REQ_add1_attr_by_NID(req,
+							  NID_ext_req,
+							  V_ASN1_SEQUENCE,
+							  extensions,
+							  extensions_len);
+				talloc_free(extensions);
+			}
+			if (entry->cm_cert_nickname != NULL) {
+				nickname = entry->cm_cert_nickname;
+			} else
+			if (entry->cm_key_nickname != NULL) {
+				nickname = entry->cm_key_nickname;
 			} else {
-				cm_log(1,
-				       "Error converting template certificate "
-				       "into a CSR.\n");
-				while ((error = ERR_get_error()) != 0) {
-					ERR_error_string_n(error, buf,
-							   sizeof(buf));
-					cm_log(1, "%s\n", buf);
-				}
-				_exit(CM_STATUS_ERROR_INTERNAL);
+				nickname = entry->cm_nickname;
+			}
+			if ((nickname != NULL) &&
+			    (cm_store_utf8_to_bmp_string(nickname, &bmp,
+							 &bmpcount) == 0)) {
+				X509_REQ_add1_attr_by_NID(req,
+							  NID_friendlyName,
+							  V_ASN1_BMPSTRING,
+							  bmp,
+							  bmpcount);
+				free(bmp);
+			}
+			password = entry->cm_challenge_password;
+			upassword = (unsigned char *) password;
+			if (password != NULL) {
+				X509_REQ_add1_attr_by_NID(req,
+							  NID_pkcs9_challengePassword,
+							  V_ASN1_PRINTABLESTRING,
+							  upassword,
+							  strlen(password));
+			}
+			X509_REQ_sign(req, pkey, cm_prefs_ossl_hash());
+			PEM_write_X509_REQ_NEW(status, req);
+			memset(&spkac, 0, sizeof(spkac));
+			spkac.challenge = M_ASN1_IA5STRING_new();
+			if (entry->cm_challenge_password != NULL) {
+				ASN1_STRING_set(spkac.challenge,
+						entry->cm_challenge_password,
+						strlen(entry->cm_challenge_password));
+			} else {
+				ASN1_STRING_set(spkac.challenge,
+						"", 0);
+			}
+			memset(&spki, 0, sizeof(spki));
+			spki.spkac = &spkac;
+			spki.sig_algor = req->sig_alg;
+			spki.signature = M_ASN1_BIT_STRING_new();
+			NETSCAPE_SPKI_set_pubkey(&spki, pkey);
+			NETSCAPE_SPKI_sign(&spki, pkey, cm_prefs_ossl_hash());
+			s = NETSCAPE_SPKI_b64_encode(&spki);
+			if (s != NULL) {
+				fprintf(status, "%s\n", s);
 			}
 		} else {
 			cm_log(1, "Error creating template certificate.\n");
@@ -237,7 +254,7 @@ cm_csrgen_o_main(int fd, struct cm_store_ca *ca, struct cm_store_entry *entry,
 				ERR_error_string_n(error, buf, sizeof(buf));
 				cm_log(1, "%s\n", buf);
 			}
-			_exit(CM_STATUS_ERROR_INTERNAL);
+			_exit(CM_SUB_STATUS_INTERNAL_ERROR);
 		}
 	}
 	while ((error = ERR_get_error()) != 0) {
@@ -306,7 +323,7 @@ cm_csrgen_o_need_pin(struct cm_store_entry *entry,
 	int status;
 	status = cm_subproc_get_exitstatus(entry, state->subproc);
 	if (WIFEXITED(status) &&
-	    (WEXITSTATUS(status) == CM_STATUS_ERROR_AUTH)) {
+	    (WEXITSTATUS(status) == CM_SUB_STATUS_ERROR_AUTH)) {
 		return 0;
 	}
 	return -1;
@@ -320,7 +337,7 @@ cm_csrgen_o_need_token(struct cm_store_entry *entry,
 	int status;
 	status = cm_subproc_get_exitstatus(entry, state->subproc);
 	if (WIFEXITED(status) &&
-	    (WEXITSTATUS(status) == CM_STATUS_ERROR_NO_TOKEN)) {
+	    (WEXITSTATUS(status) == CM_SUB_STATUS_ERROR_NO_TOKEN)) {
 		return 0;
 	}
 	return -1;
