@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011,2012 Red Hat, Inc.
+ * Copyright (C) 2011,2012,2013,2014 Red Hat, Inc.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,15 +33,18 @@
 #include <dbus/dbus.h>
 
 #include "env.h"
+#include "log.h"
 #include "tdbus.h"
 
 static char *
 cm_env_homedir(const char *subdir, const char *subfile)
 {
 	struct passwd *pwd;
-	const char *home;
+	char *home;
 	char *ret;
 	int len;
+	dbus_bool_t free_home;
+
 	if ((subdir == NULL) && (subfile == NULL)) {
 		return NULL;
 	}
@@ -52,6 +55,11 @@ cm_env_homedir(const char *subdir, const char *subfile)
 			home = pwd->pw_name;
 		}
 	}
+	free_home = FALSE;
+	if (home != NULL) {
+		home = realpath(home, NULL);
+		free_home = (home != NULL);
+	}
 	if (home != NULL) {
 		len = strlen(home);
 		if (subdir != NULL) {
@@ -75,6 +83,9 @@ cm_env_homedir(const char *subdir, const char *subfile)
 	} else {
 		ret = NULL;
 	}
+	if (free_home) {
+		free(home);
+	}
 	return ret;
 }
 
@@ -82,7 +93,7 @@ static void
 cm_env_ensure_dir(char *path)
 {
 	char *p, *q, *tmp;
-	struct stat st;
+	int i;
 
 	if (path != NULL) {
 		tmp = strdup(path);
@@ -91,9 +102,14 @@ cm_env_ensure_dir(char *path)
 			for (q = tmp + 1; q < p; q++) {
 				if (*q == '/') {
 					*q = '\0';
-					if ((stat(tmp, &st) == -1) &&
-					    (errno == ENOENT)) {
-						mkdir(tmp, S_IRWXU);
+					i = mkdir(tmp, S_IRWXU);
+					if ((i != 0) && 
+					    (errno != EEXIST)) {
+						cm_log(0, "Error ensuring "
+						       "that directory '%s' "
+						       "exists: %s.\n", tmp,
+						       strerror(errno));
+						_exit(1);
 					}
 					*q = '/';
 				}
