@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009,2010,2011,2012 Red Hat, Inc.
+ * Copyright (C) 2009,2010,2011,2012,2013,2014 Red Hat, Inc.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,6 +36,7 @@
 #include <krb5.h>
 
 #include "cm.h"
+#include "kudict.h"
 #include "oiddict.h"
 #include "store.h"
 #include "store-int.h"
@@ -106,16 +107,28 @@ static int
 ensure_path_is_directory(char *path)
 {
 	struct stat st;
+	int err;
+
 	if (stat(path, &st) == 0) {
 		if (S_ISDIR(st.st_mode)) {
-			return 0;
+			if (access(path, R_OK | W_OK) == 0) {
+				return 0;
+			} else {
+				err = errno;
+				printf(_("Path \"%s\": insufficient "
+					 "permissions.\n"), path);
+				errno = err;
+				return -1;
+			}
 		} else {
 			printf(_("Path \"%s\" is not a directory.\n"),
 			       path);
 			return -1;
 		}
 	} else {
+		err = errno;
 		printf(_("Path \"%s\": %s.\n"), path, strerror(errno));
+		errno = err;
 		return -1;
 	}
 }
@@ -545,13 +558,14 @@ request(const char *argv0, int argc, char **argv)
 	enum cm_tdbus_type bus = CM_DBUS_DEFAULT_BUS;
 	char subject_default[LINE_MAX];
 	char *nss_scheme, *dbdir = NULL, *token = NULL, *nickname = NULL;
-	char *keyfile = NULL, *certfile = NULL, *capath;
+	char *keytype = NULL, *keyfile = NULL, *certfile = NULL, *capath;
 	char *pin = NULL, *pinfile = NULL;
-	int keysize = 0, auto_renew = 1, verbose = 0, c, i;
+	int keysize = 0, auto_renew = 1, verbose = 0, ku = 0, kubit, c, i, j;
 	char *ca = DEFAULT_CA, *subject = NULL, **eku = NULL, *oid, *id = NULL;
+	char *profile = NULL, kustring[16];
 	char **principal = NULL, **dns = NULL, **email = NULL;
-	struct cm_tdbusm_dict param[34];
-	const struct cm_tdbusm_dict *params[35];
+	struct cm_tdbusm_dict param[38];
+	const struct cm_tdbusm_dict *params[37];
 	DBusMessage *req, *rep;
 	dbus_bool_t b;
 	char *p;
@@ -582,7 +596,7 @@ request(const char *argv0, int argc, char **argv)
 
 	opterr = 0;
 	while ((c = getopt(argc, argv,
-			   ":d:n:t:k:f:I:g:rRN:U:K:D:E:sSp:P:vB:C:"
+			   ":d:n:t:k:f:I:g:rRN:u:U:K:D:E:sSp:P:vB:C:T:G:"
 			   GETOPT_CA)) != -1) {
 		switch (c) {
 		case 'd':
@@ -605,6 +619,31 @@ request(const char *argv0, int argc, char **argv)
 		case 'f':
 			certfile = ensure_pem(globals.tctx, optarg);
 			break;
+		case 'G':
+			if ((strcasecmp(optarg, "RSA") != 0)
+#ifdef CM_ENABLE_DSA
+			    && (strcasecmp(optarg, "DSA") != 0)
+#endif
+#ifdef CM_ENABLE_EC
+			    && (strcasecmp(optarg, "ECDSA") != 0)
+			    && (strcasecmp(optarg, "EC") != 0)
+#endif
+			    ) {
+				printf(_("No support for generating \"%s\" keys.\n"),
+				       optarg);
+				printf(_("Known key types include:"));
+				printf(" RSA");
+#ifdef CM_ENABLE_DSA
+				printf(" DSA");
+#endif
+#ifdef CM_ENABLE_EC
+				printf(" EC");
+#endif
+				printf("\n");
+				return 1;
+			}
+			keytype = talloc_strdup(globals.tctx, optarg);
+			break;
 		case 'g':
 			keysize = atoi(optarg);
 			break;
@@ -620,9 +659,21 @@ request(const char *argv0, int argc, char **argv)
 		case 'c':
 			ca = talloc_strdup(globals.tctx, optarg);
 			break;
+		case 'T':
+			profile = talloc_strdup(globals.tctx, optarg);
+			break;
 		case 'N':
 			subject = talloc_strdup(globals.tctx, optarg);
 			break;
+		case 'u':
+			kubit = cm_ku_from_name(optarg);
+			if (kubit == -1) {
+				printf(_("Unrecognized keyUsage \"%s\".\n"),
+				       optarg);
+				return 1;
+			}
+			ku |= (1 << kubit);
+			break;
 		case 'U':
 			oid = cm_oid_from_name(globals.tctx, optarg);
 			if ((oid == NULL) ||
@@ -859,12 +910,14 @@ request(const char *argv0, int argc, char **argv)
 	param[i].value.b = auto_renew > 0;
 	params[i] = &param[i];
 	i++;
-	if (keysize > 0) {
+	if (keytype != NULL) {
 		param[i].key = "KEY_TYPE";
 		param[i].value_type = cm_tdbusm_dict_s;
-		param[i].value.s = "RSA";
+		param[i].value.s = keytype;
 		params[i] = &param[i];
 		i++;
+	}
+	if (keysize > 0) {
 		param[i].key = "KEY_SIZE";
 		param[i].value_type = cm_tdbusm_dict_n;
 		param[i].value.n = keysize;
@@ -918,6 +971,17 @@ request(const char *argv0, int argc, char **argv)
 		params[i] = &param[i];
 		i++;
 	}
+	if (ku != 0) {
+		for (j = 0; (ku >> j) != 0; j++) {
+			kustring[j] = ((ku >> j) & 1) ? '1' : '0';
+		}
+		kustring[j] = '\0';
+		param[i].key = "KU";
+		param[i].value_type = cm_tdbusm_dict_s;
+		param[i].value.s = kustring;
+		params[i] = &param[i];
+		i++;
+	}
 	if (eku != NULL) {
 		param[i].key = "EKU";
 		param[i].value_type = cm_tdbusm_dict_as;
@@ -925,6 +989,13 @@ request(const char *argv0, int argc, char **argv)
 		params[i] = &param[i];
 		i++;
 	}
+	if (profile != NULL) {
+		param[i].key = CM_DBUS_PROP_TEMPLATE_PROFILE;
+		param[i].value_type = cm_tdbusm_dict_s;
+		param[i].value.s = profile;
+		params[i] = &param[i];
+		i++;
+	}
 	if (precommand != NULL) {
 		param[i].key = CM_DBUS_PROP_CERT_PRESAVE_COMMAND;
 		param[i].value_type = cm_tdbusm_dict_s;
@@ -1096,13 +1167,14 @@ add_basic_request(enum cm_tdbus_type bus, char *id,
 		  char *dbdir, char *nickname, char *token,
 		  char *keyfile, char *certfile,
 		  char *pin, char *pinfile,
-		  char *ca, char *precommand, char *postcommand,
+		  char *ca, char *profile,
+		  char *precommand, char *postcommand,
 		  dbus_bool_t auto_renew_stop, int verbose)
 {
 	DBusMessage *req, *rep;
 	int i;
-	struct cm_tdbusm_dict param[21];
-	const struct cm_tdbusm_dict *params[22];
+	struct cm_tdbusm_dict param[22];
+	const struct cm_tdbusm_dict *params[23];
 	dbus_bool_t b;
 	const char *capath;
 	char *p;
@@ -1208,6 +1280,13 @@ add_basic_request(enum cm_tdbus_type bus, char *id,
 	param[i].value.b = !auto_renew_stop;
 	params[i] = &param[i];
 	i++;
+	if (profile != NULL) {
+		param[i].key = CM_DBUS_PROP_TEMPLATE_PROFILE;
+		param[i].value_type = cm_tdbusm_dict_s;
+		param[i].value.s = profile;
+		params[i] = &param[i];
+		i++;
+	}
 	if (precommand != NULL) {
 		param[i].key = CM_DBUS_PROP_CERT_PRESAVE_COMMAND;
 		param[i].value_type = cm_tdbusm_dict_s;
@@ -1267,15 +1346,17 @@ set_tracking(const char *argv0, const char *category,
 	enum cm_tdbus_type bus = CM_DBUS_DEFAULT_BUS;
 	DBusMessage *req, *rep;
 	const char *request, *capath;
-	struct cm_tdbusm_dict param[12];
-	const struct cm_tdbusm_dict *params[13];
+	struct cm_tdbusm_dict param[14];
+	const struct cm_tdbusm_dict *params[15];
 	char *nss_scheme, *dbdir = NULL, *token = NULL, *nickname = NULL;
 	char *id = NULL, *new_id = NULL, *new_request;
 	char *keyfile = NULL, *certfile = NULL, *ca = DEFAULT_CA;
+	char *profile = NULL;
 	char *pin = NULL, *pinfile = NULL;
 	dbus_bool_t b;
-	int c, auto_renew_start = 0, auto_renew_stop = 0, verbose = 0, i;
-	char **eku = NULL, *oid;
+	int c, auto_renew_start = 0, auto_renew_stop = 0, verbose = 0, i, j;
+	int ku = 0, kubit;
+	char **eku = NULL, *oid, kustring[16];
 	char **principal = NULL, **dns = NULL, **email = NULL;
 	krb5_context kctx;
 	krb5_error_code kret;
@@ -1297,7 +1378,7 @@ set_tracking(const char *argv0, const char *category,
 
 	opterr = 0;
 	while ((c = getopt(argc, argv,
-			   ":d:n:t:k:f:g:p:P:rRi:I:U:K:D:E:sSvB:C:"
+			   ":d:n:t:k:f:g:p:P:rRi:I:u:U:K:D:E:sSvB:C:T:"
 			   GETOPT_CA)) != -1) {
 		switch (c) {
 		case 'd':
@@ -1344,12 +1425,24 @@ set_tracking(const char *argv0, const char *category,
 				return 1;
 			}
 			break;
+		case 'T':
+			profile = talloc_strdup(globals.tctx, optarg);
+			break;
 		case 'i':
 			id = talloc_strdup(globals.tctx, optarg);
 			break;
 		case 'I':
 			new_id = talloc_strdup(globals.tctx, optarg);
 			break;
+		case 'u':
+			kubit = cm_ku_from_name(optarg);
+			if (kubit == -1) {
+				printf(_("Unrecognized keyUsage \"%s\".\n"),
+				       optarg);
+				return 1;
+			}
+			ku |= (1 << kubit);
+			break;
 		case 'U':
 			oid = cm_oid_from_name(globals.tctx, optarg);
 			if ((oid == NULL) ||
@@ -1501,6 +1594,18 @@ set_tracking(const char *argv0, const char *category,
 				params[i] = &param[i];
 				i++;
 			}
+			if (ku != 0) {
+				for (j = 0; (ku >> j) != 0; j++) {
+					kustring[j] = ((ku >> j) & 1) ?
+						      '1' : '0';
+				}
+				kustring[j] = '\0';
+				param[i].key = "KU";
+				param[i].value_type = cm_tdbusm_dict_s;
+				param[i].value.s = kustring;
+				params[i] = &param[i];
+				i++;
+			}
 			if (eku != NULL) {
 				param[i].key = "EKU";
 				param[i].value_type = cm_tdbusm_dict_as;
@@ -1546,6 +1651,13 @@ set_tracking(const char *argv0, const char *category,
 			} else {
 				capath = NULL;
 			}
+			if (profile != NULL) {
+				param[i].key = CM_DBUS_PROP_TEMPLATE_PROFILE;
+				param[i].value_type = cm_tdbusm_dict_s;
+				param[i].value.s = profile;
+				params[i] = &param[i];
+				i++;
+			}
 			if (precommand != NULL) {
 				param[i].key = CM_DBUS_PROP_CERT_PRESAVE_COMMAND;
 				param[i].value_type = cm_tdbusm_dict_s;
@@ -1621,7 +1733,7 @@ set_tracking(const char *argv0, const char *category,
 						 dbdir, nickname, token,
 						 keyfile, certfile,
 						 pin, pinfile,
-						 ca,
+						 ca, profile,
 						 precommand, postcommand,
 						 (auto_renew_stop > 0),
 						 verbose);
@@ -1686,15 +1798,16 @@ resubmit(const char *argv0, int argc, char **argv)
 	DBusMessage *req, *rep;
 	const char *request;
 	char *capath;
-	struct cm_tdbusm_dict param[17];
-	const struct cm_tdbusm_dict *params[18];
+	struct cm_tdbusm_dict param[18];
+	const struct cm_tdbusm_dict *params[19];
 	char *dbdir = NULL, *token = NULL, *nickname = NULL, *certfile = NULL;
 	char *pin = NULL, *pinfile = NULL;
 	char *id = NULL, *new_id = NULL, *ca = NULL, *new_request, *nss_scheme;
 	char *subject = NULL, **eku = NULL, *oid = NULL;
 	char **principal = NULL, **dns = NULL, **email = NULL;
+	char *profile = NULL, kustring[16];
 	dbus_bool_t b;
-	int verbose = 0, c, i;
+	int verbose = 0, ku = 0, kubit, c, i, j;
 	krb5_context kctx;
 	krb5_error_code kret;
 	krb5_principal kprincipal;
@@ -1710,7 +1823,8 @@ resubmit(const char *argv0, int argc, char **argv)
 
 	opterr = 0;
 	while ((c = getopt(argc, argv,
-			   ":d:n:N:t:U:K:E:D:f:i:I:sSp:P:vB:C:" GETOPT_CA)) != -1) {
+			   ":d:n:N:t:u:U:K:E:D:f:i:I:sSp:P:vB:C:T:"
+			   GETOPT_CA)) != -1) {
 		switch (c) {
 		case 'd':
 			nss_scheme = NULL;
@@ -1732,6 +1846,9 @@ resubmit(const char *argv0, int argc, char **argv)
 		case 'c':
 			ca = talloc_strdup(globals.tctx, optarg);
 			break;
+		case 'T':
+			profile = talloc_strdup(globals.tctx, optarg);
+			break;
 		case 'i':
 			id = talloc_strdup(globals.tctx, optarg);
 			break;
@@ -1741,6 +1858,15 @@ resubmit(const char *argv0, int argc, char **argv)
 		case 'N':
 			subject = talloc_strdup(globals.tctx, optarg);
 			break;
+		case 'u':
+			kubit = cm_ku_from_name(optarg);
+			if (kubit == -1) {
+				printf(_("Unrecognized keyUsage \"%s\".\n"),
+				       optarg);
+				return 1;
+			}
+			ku |= (1 << kubit);
+			break;
 		case 'U':
 			oid = cm_oid_from_name(globals.tctx, optarg);
 			if ((oid == NULL) ||
@@ -1906,6 +2032,18 @@ resubmit(const char *argv0, int argc, char **argv)
 		params[i] = &param[i];
 		i++;
 	}
+	if (ku != 0) {
+		for (j = 0; (ku >> j) != 0; j++) {
+			kustring[j] = ((ku >> j) & 1) ?
+				      '1' : '0';
+		}
+		kustring[j] = '\0';
+		param[i].key = "KU";
+		param[i].value_type = cm_tdbusm_dict_s;
+		param[i].value.s = kustring;
+		params[i] = &param[i];
+		i++;
+	}
 	if (eku != NULL) {
 		param[i].key = "EKU";
 		param[i].value_type = cm_tdbusm_dict_as;
@@ -1927,6 +2065,13 @@ resubmit(const char *argv0, int argc, char **argv)
 		params[i] = &param[i];
 		i++;
 	}
+	if (profile != NULL) {
+		param[i].key = CM_DBUS_PROP_TEMPLATE_PROFILE;
+		param[i].value_type = cm_tdbusm_dict_s;
+		param[i].value.s = profile;
+		params[i] = &param[i];
+		i++;
+	}
 	if (precommand != NULL) {
 		param[i].key = CM_DBUS_PROP_CERT_PRESAVE_COMMAND;
 		param[i].value_type = cm_tdbusm_dict_s;
@@ -2008,8 +2153,10 @@ list(const char *argv0, int argc, char **argv)
 	dbus_bool_t b;
 	char *s1, *s2, *s3, *s4, *s5, *s6;
 	long n1, n2;
-	char **as1, **as2, **as3, **as4, t[24];
+	char **as1, **as2, **as3, **as4, t[25];
 	int requests_only = 0, tracking_only = 0, verbose = 0, c, i, j;
+	unsigned int k;
+	char key_usages[LINE_MAX];
 
 	opterr = 0;
 	while ((c = getopt(argc, argv, ":rtsSvd:n:f:i:" GETOPT_CA)) != -1) {
@@ -2134,6 +2281,7 @@ list(const char *argv0, int argc, char **argv)
 			continue;
 			break;
 		case CM_NEED_KEY_PAIR:
+		case CM_NEED_KEY_GEN_PERMS:
 		case CM_NEED_KEY_GEN_PIN:
 		case CM_NEED_KEY_GEN_TOKEN:
 		case CM_GENERATING_KEY_PAIR:
@@ -2154,6 +2302,7 @@ list(const char *argv0, int argc, char **argv)
 		case CM_PRE_SAVE_CERT:
 		case CM_START_SAVING_CERT:
 		case CM_SAVING_CERT:
+		case CM_NEED_CERTSAVE_PERMS:
 		case CM_SAVED_CERT:
 		case CM_POST_SAVED_CERT:
 		case CM_NEED_TO_READ_CERT:
@@ -2177,8 +2326,14 @@ list(const char *argv0, int argc, char **argv)
 			}
 			break;
 		case CM_MONITORING:
-		case CM_NEED_TO_NOTIFY:
-		case CM_NOTIFYING:
+		case CM_NEED_TO_NOTIFY_VALIDITY:
+		case CM_NOTIFYING_VALIDITY:
+		case CM_NEED_TO_NOTIFY_REJECTION:
+		case CM_NOTIFYING_REJECTION:
+		case CM_NEED_TO_NOTIFY_ISSUED_FAILED:
+		case CM_NOTIFYING_ISSUED_FAILED:
+		case CM_NEED_TO_NOTIFY_ISSUED_SAVED:
+		case CM_NOTIFYING_ISSUED_SAVED:
 			if (requests_only) {
 				continue;
 			}
@@ -2317,6 +2472,23 @@ list(const char *argv0, int argc, char **argv)
 			       as3[j],
 			       as3[j + 1] ? "" : "\n");
 		}
+		if (n2 != 0) {
+			const char *ku;
+			memset(key_usages, '\0', sizeof(key_usages));
+			for (k = 0; (n2 >> k) != 0; k++) {
+				if ((((n2 >> k) & 1) != 0) &&
+				    ((ku = cm_ku_to_name(k)) != NULL)) {
+					snprintf(key_usages +
+						 strlen(key_usages),
+						 sizeof(key_usages) -
+						 strlen(key_usages),
+						 "%s%s",
+						 strlen(key_usages) ? "," : "",
+						 ku);
+				}
+			}
+			printf(_("\tkey usage: %s\n"), key_usages);
+		}
 		for (j = 0; (as4 != NULL) && (as4[j] != NULL); j++) {
 			printf("%s%s%s",
 			       j == 0 ? _("\teku: ") : ",",
@@ -2463,15 +2635,18 @@ help(const char *cmd, const char *category)
 		N_("Optional arguments:\n"),
 		N_("* Certificate handling settings:\n"),
 		N_("  -I NAME	nickname to assign to the request\n"),
+		N_("  -G TYPE	type of key to be generated if one is not already in place\n"),
 		N_("  -g SIZE	size of key to be generated if one is not already in place\n"),
 		N_("  -r		attempt to renew the certificate when expiration nears (default)\n"),
 		N_("  -R		don't attempt to renew the certificate when expiration nears\n"),
 #ifndef FORCE_CA
 		N_("  -c CA		use the specified CA rather than the default\n"),
 #endif
+		N_("  -T PROFILE	ask the CA to process the request using the named profile or template\n"),
 		N_("* Parameters for the signing request:\n"),
 		N_("  -N NAME	set requested subject name (default: CN=<hostname>)\n"),
 		N_("  -U EXTUSAGE	set requested extended key usage OID\n"),
+		N_("  -u KEYUSAGE	set requested key usage value\n"),
 		N_("  -K NAME	set requested principal name\n"),
 		N_("  -D DNSNAME	set requested DNS name\n"),
 		N_("  -E EMAIL	set requested email address\n"),
@@ -2509,8 +2684,10 @@ help(const char *cmd, const char *category)
 #ifndef FORCE_CA
 		N_("  -c CA		use the specified CA rather than the default\n"),
 #endif
+		N_("  -T PROFILE	ask the CA to process the request using the named profile or template\n"),
 		N_("* Parameters for the signing request at renewal time:\n"),
 		N_("  -U EXTUSAGE	override requested extended key usage OID\n"),
+		N_("  -u KEYUSAGE	set requested key usage value\n"),
 		N_("  -K NAME	override requested principal name\n"),
 		N_("  -D DNSNAME	override requested DNS name\n"),
 		N_("  -E EMAIL	override requested email address\n"),
@@ -2565,6 +2742,7 @@ help(const char *cmd, const char *category)
 		N_("* New parameter values for the signing request:\n"),
 		N_("  -N NAME	set requested subject name (default: CN=<hostname>)\n"),
 		N_("  -U EXTUSAGE	set requested extended key usage OID\n"),
+		N_("  -u KEYUSAGE	set requested key usage value\n"),
 		N_("  -K NAME	set requested principal name\n"),
 		N_("  -D DNSNAME	set requested DNS name\n"),
 		N_("  -E EMAIL	set requested email address\n"),
@@ -2575,6 +2753,7 @@ help(const char *cmd, const char *category)
 #ifndef FORCE_CA
 		N_("  -c CA		use the specified CA rather than the current one\n"),
 #endif
+		N_("  -T PROFILE	ask the CA to process the request using the named profile or template\n"),
 		N_("* Bus options:\n"),
 		N_("  -S		connect to the certmonger service on the system bus\n"),
 		N_("  -s		connect to the certmonger service on the session bus\n"),
