@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009,2010,2011,2012 Red Hat, Inc.
+ * Copyright (C) 2009,2010,2011,2012,2013,2014 Red Hat, Inc.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -70,6 +70,9 @@ cm_entry_reset_state(struct cm_store_entry *entry)
 	case CM_NEED_KEY_GEN_PIN:
 		entry->cm_state = CM_NEED_KEY_PAIR;
 		break;
+	case CM_NEED_KEY_GEN_PERMS:
+		entry->cm_state = CM_NEED_KEY_PAIR;
+		break;
 	case CM_HAVE_KEY_PAIR:
 		break;
 	case CM_NEED_KEYINFO:
@@ -116,6 +119,9 @@ cm_entry_reset_state(struct cm_store_entry *entry)
 	case CM_SAVING_CERT:
 		entry->cm_state = CM_NEED_TO_SAVE_CERT;
 		break;
+	case CM_NEED_CERTSAVE_PERMS:
+		entry->cm_state = CM_NEED_TO_SAVE_CERT;
+		break;
 	case CM_NEED_TO_READ_CERT:
 		break;
 	case CM_READING_CERT:
@@ -124,7 +130,7 @@ cm_entry_reset_state(struct cm_store_entry *entry)
 	case CM_SAVED_CERT:
 		break;
 	case CM_POST_SAVED_CERT:
-		entry->cm_state = CM_POST_SAVED_CERT;
+		entry->cm_state = CM_SAVED_CERT;
 		break;
 	case CM_CA_REJECTED:
 		break;
@@ -144,31 +150,49 @@ cm_entry_reset_state(struct cm_store_entry *entry)
 		break;
 	case CM_MONITORING:
 		break;
-	case CM_NEED_TO_NOTIFY:
+	case CM_NEED_TO_NOTIFY_VALIDITY:
 		entry->cm_state = CM_MONITORING;
 		break;
-	case CM_NOTIFYING:
-		entry->cm_state = CM_NEED_TO_NOTIFY;
+	case CM_NOTIFYING_VALIDITY:
+		entry->cm_state = CM_NEED_TO_NOTIFY_VALIDITY;
+		break;
+	case CM_NEED_TO_NOTIFY_REJECTION:
+		break;
+	case CM_NOTIFYING_REJECTION:
+		entry->cm_state = CM_NEED_TO_NOTIFY_REJECTION;
+		break;
+	case CM_NEED_TO_NOTIFY_ISSUED_FAILED:
+		break;
+	case CM_NOTIFYING_ISSUED_FAILED:
+		entry->cm_state = CM_NEED_TO_NOTIFY_ISSUED_FAILED;
+		break;
+	case CM_NEED_TO_NOTIFY_ISSUED_SAVED:
+		break;
+	case CM_NOTIFYING_ISSUED_SAVED:
+		entry->cm_state = CM_NEED_TO_NOTIFY_ISSUED_SAVED;
 		break;
 	case CM_NEWLY_ADDED:
 		break;
 	case CM_NEWLY_ADDED_START_READING_KEYINFO:
+		entry->cm_state = CM_NEWLY_ADDED;
 		break;
 	case CM_NEWLY_ADDED_READING_KEYINFO:
-		entry->cm_state = CM_NEWLY_ADDED_START_READING_KEYINFO;
+		entry->cm_state = CM_NEWLY_ADDED;
 		break;
 	case CM_NEWLY_ADDED_NEED_KEYINFO_READ_TOKEN:
-		entry->cm_state = CM_NEWLY_ADDED_START_READING_KEYINFO;
+		entry->cm_state = CM_NEWLY_ADDED;
 		break;
 	case CM_NEWLY_ADDED_NEED_KEYINFO_READ_PIN:
-		entry->cm_state = CM_NEWLY_ADDED_START_READING_KEYINFO;
+		entry->cm_state = CM_NEWLY_ADDED;
 		break;
 	case CM_NEWLY_ADDED_START_READING_CERT:
+		entry->cm_state = CM_NEWLY_ADDED;
 		break;
 	case CM_NEWLY_ADDED_READING_CERT:
-		entry->cm_state = CM_NEWLY_ADDED_START_READING_CERT;
+		entry->cm_state = CM_NEWLY_ADDED;
 		break;
 	case CM_NEWLY_ADDED_DECIDING:
+		entry->cm_state = CM_NEWLY_ADDED;
 		break;
 	case CM_INVALID:
 		/* not reached */
@@ -186,7 +210,13 @@ cm_waitfor_readable_fd(int fd, int delay)
 	tv.tv_sec = delay;
 	FD_ZERO(&fds);
 	FD_SET(fd, &fds);
-	select(fd + 1, &fds, NULL, &fds, (delay >= 0) ? &tv : NULL);
+	if (select(fd + 1, &fds, NULL, &fds, (delay >= 0) ? &tv : NULL) < 0) {
+		if (delay < 0) {
+			/* No defined delay, but an error. */
+			cm_log(3, "indefinite select() on %d returned error: "
+			       "%s\n", fd, strerror(errno));
+		}
+	}
 }
 
 /* Decide how long to wait before contacting the CA again. */
@@ -219,6 +249,46 @@ cm_decide_monitor_delay(time_t remaining)
 	return delay;
 }
 
+/* Manage a "lock" that we use to serialize access to THE REST OF THE WORLD. */
+static struct cm_store_entry *writing_lock;
+static dbus_bool_t
+cm_writing_has_lock(struct cm_store_entry *entry)
+{
+	return (writing_lock == entry);
+}
+static dbus_bool_t
+cm_writing_lock(struct cm_store_entry *entry)
+{
+	if ((writing_lock == entry) || (writing_lock == NULL)) {
+		if (writing_lock == NULL) {
+			cm_log(3, "%s('%s') taking writing lock\n",
+			       entry->cm_busname, entry->cm_nickname);
+			writing_lock = entry;
+		} else {
+			abort();
+		}
+		return TRUE;
+	} else {
+		return FALSE;
+	}
+}
+static dbus_bool_t
+cm_writing_unlock(struct cm_store_entry *entry)
+{
+	if ((writing_lock == entry) || (writing_lock == NULL)) {
+		if (writing_lock == entry) {
+			cm_log(3, "%s('%s') releasing writing lock\n",
+			       entry->cm_busname, entry->cm_nickname);
+			writing_lock = NULL;
+		} else {
+			abort();
+		}
+		return TRUE;
+	} else {
+		return FALSE;
+	}
+}
+
 /* Set up run-time data associated with the entry. */
 int
 cm_iterate_init(struct cm_store_entry *entry, void **cm_iterate_state)
@@ -232,6 +302,9 @@ cm_iterate_init(struct cm_store_entry *entry, void **cm_iterate_state)
 	memset(state, 0, sizeof(*state));
 	*cm_iterate_state = state;
 	cm_entry_reset_state(entry);
+	if (cm_writing_has_lock(entry)) {
+		cm_writing_unlock(entry);
+	}
 	state->cm_keyiread_state = cm_keyiread_start(entry);
 	if (state->cm_keyiread_state != NULL) {
 		while (cm_keyiread_ready(entry,
@@ -356,6 +429,14 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 
 	switch (entry->cm_state) {
 	case CM_NEED_KEY_PAIR:
+		if (!cm_writing_lock(entry)) {
+			/* Just hang out in this state while we're messing
+			 * around with the outside world for another entry. */
+			cm_log(3, "%s('%s') waiting for saving lock\n",
+			       entry->cm_busname, entry->cm_nickname);
+			*when = cm_time_soon;
+			break;
+		}
 		/* Start a helper. */
 		state->cm_keygen_state = cm_keygen_start(entry);
 		if (state->cm_keygen_state != NULL) {
@@ -371,12 +452,23 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 			}
 		} else {
 			/* Failed to start generating a key; try again. */
+			cm_writing_unlock(entry);
 			*when = cm_time_soonish;
 		}
 		break;
 
 	case CM_GENERATING_KEY_PAIR:
 		if (cm_keygen_ready(entry, state->cm_keygen_state) == 0) {
+			if (!cm_writing_unlock(entry)) {
+				/* If for some reason we fail to release the
+				 * lock that we have, try to release it again
+				 * soon. */
+				*when = cm_time_soon;
+				cm_log(1, "%s('%s') failed to release saving "
+				       "lock, probably a bug\n",
+				       entry->cm_busname, entry->cm_nickname);
+				break;
+			}
 			if (cm_keygen_saved_keypair(entry,
 						    state->cm_keygen_state) == 0) {
 				/* Saved key pair; move on. */
@@ -385,6 +477,14 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 				entry->cm_state = CM_HAVE_KEY_PAIR;
 				*when = cm_time_now;
 			} else
+			if (cm_keygen_need_perms(entry,
+					         state->cm_keygen_state) == 0) {
+				/* Whoops, we need help. */
+				cm_keygen_done(entry, state->cm_keygen_state);
+				state->cm_keygen_state = NULL;
+				entry->cm_state = CM_NEED_KEY_GEN_PERMS;
+				*when = cm_time_now;
+			} else
 			if (cm_keygen_need_token(entry,
 					         state->cm_keygen_state) == 0) {
 				/* Whoops, we need help. */
@@ -420,6 +520,11 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 		}
 		break;
 
+	case CM_NEED_KEY_GEN_PERMS:
+		/* Revisit this later. */
+		*when = cm_time_no_time;
+		break;
+
 	case CM_NEED_KEY_GEN_TOKEN:
 		/* Revisit this later. */
 		*when = cm_time_no_time;
@@ -635,6 +740,8 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 					     state->cm_submit_state) == 0) {
 				/* We're all done.  Save the certificate to its
 				 * real home. */
+				cm_submit_clear_ca_cookie(entry,
+							  state->cm_submit_state);
 				cm_submit_done(entry, state->cm_submit_state);
 				state->cm_submit_state = NULL;
 				entry->cm_state = CM_NEED_TO_SAVE_CERT;
@@ -643,6 +750,8 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 			if (cm_submit_rejected(entry,
 					       state->cm_submit_state) == 0) {
 				/* The request was flat-out rejected. */
+				cm_submit_clear_ca_cookie(entry,
+							  state->cm_submit_state);
 				cm_submit_done(entry, state->cm_submit_state);
 				state->cm_submit_state = NULL;
 				if (entry->cm_cert != NULL) {
@@ -654,13 +763,14 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 					entry->cm_state = CM_MONITORING;
 					*when = cm_time_soonish;
 				} else {
-					entry->cm_state = CM_CA_REJECTED;
+					entry->cm_state = CM_NEED_TO_NOTIFY_REJECTION;
 					*when = cm_time_now;
 				}
 			} else
 			if (cm_submit_unreachable(entry,
 						  state->cm_submit_state) == 0) {
-				/* Let's try again later. */
+				/* Let's try again later.  The cookie is left
+				 * unmodified. */
 				*delay = cm_submit_specified_delay(entry,
 								   state->cm_submit_state);
 				cm_submit_done(entry, state->cm_submit_state);
@@ -734,6 +844,14 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 		break;
 
 	case CM_NEED_TO_SAVE_CERT:
+		if (!cm_writing_lock(entry)) {
+			/* Just hang out in this state while we're messing
+			 * around with the outside world for another entry. */
+			cm_log(3, "%s('%s') waiting for saving lock\n",
+			       entry->cm_busname, entry->cm_nickname);
+			*when = cm_time_soon;
+			break;
+		}
 		if (entry->cm_pre_certsave_command != NULL) {
 			state->cm_hook_state = cm_hook_start_presave(entry);
 			if (state->cm_hook_state != NULL) {
@@ -805,12 +923,22 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 				state->cm_certsave_state = NULL;
 				entry->cm_state = CM_NEED_TO_READ_CERT;
 				*when = cm_time_now;
-			} else {
-				/* Failed to save cert; try again in a bit. */
+			} else
+			if (cm_certsave_permissions_error(entry,
+							  state->cm_certsave_state) == 0) {
+				/* Whoops, we need help. */
 				cm_certsave_done(entry,
 						 state->cm_certsave_state);
 				state->cm_certsave_state = NULL;
-				entry->cm_state = CM_NEED_TO_SAVE_CERT;
+				entry->cm_state = CM_NEED_CERTSAVE_PERMS;
+				*when = cm_time_now;
+			} else {
+				/* Failed to save cert; make a note and try
+				 * again in a bit. */
+				cm_certsave_done(entry,
+						 state->cm_certsave_state);
+				state->cm_certsave_state = NULL;
+				entry->cm_state = CM_NEED_TO_NOTIFY_ISSUED_FAILED;
 				*when = cm_time_soonish;
 			}
 		} else {
@@ -825,7 +953,23 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 		}
 		break;
 
+	case CM_NEED_CERTSAVE_PERMS:
+		/* Revisit this later. */
+		*when = cm_time_no_time;
+		break;
+
 	case CM_NEED_TO_READ_CERT:
+		/* We should already have the lock here.  In cases where we're
+		 * resuming things at startup, try to acquire it if we don't
+		 * have it. */
+		if (!cm_writing_has_lock(entry) && !cm_writing_lock(entry)) {
+			/* Just hang out in this state while we're messing
+			 * around with the outside world for another entry. */
+			cm_log(3, "%s('%s') waiting for saving lock\n",
+			       entry->cm_busname, entry->cm_nickname);
+			*when = cm_time_soon;
+			break;
+		}
 		state->cm_certread_state = cm_certread_start(entry);
 		if (state->cm_certread_state != NULL) {
 			/* Note that we're reading the cert. */
@@ -868,6 +1012,17 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 		break;
 
 	case CM_SAVED_CERT:
+		/* We should already have the lock here.  In cases where we're
+		 * resuming things at startup, try to acquire it if we don't
+		 * have it. */
+		if (!cm_writing_has_lock(entry) && !cm_writing_lock(entry)) {
+			/* Just hang out in this state while we're messing
+			 * around with the outside world for another entry. */
+			cm_log(3, "%s('%s') waiting for saving lock\n",
+			       entry->cm_busname, entry->cm_nickname);
+			*when = cm_time_soon;
+			break;
+		}
 		if (entry->cm_post_certsave_command != NULL) {
 			state->cm_hook_state = cm_hook_start_postsave(entry);
 			if (state->cm_hook_state != NULL) {
@@ -883,11 +1038,11 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 				}
 			} else {
 				/* Failed to start the post-save; skip it. */
-				entry->cm_state = CM_MONITORING;
+				entry->cm_state = CM_NEED_TO_NOTIFY_ISSUED_SAVED;
 				*when = cm_time_soon;
 			}
 		} else {
-			entry->cm_state = CM_MONITORING;
+			entry->cm_state = CM_NEED_TO_NOTIFY_ISSUED_SAVED;
 			*when = cm_time_now;
 		}
 		break;
@@ -896,7 +1051,7 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 		if (cm_hook_ready(entry, state->cm_hook_state) == 0) {
 			cm_hook_done(entry, state->cm_hook_state);
 			state->cm_hook_state = NULL;
-			entry->cm_state = CM_MONITORING;
+			entry->cm_state = CM_NEED_TO_NOTIFY_ISSUED_SAVED;
 			*when = cm_time_now;
 		} else {
 			/* Wait for status update, or poll. */
@@ -942,7 +1097,7 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 						       &cm_prefs_notify_ttls,
 						       &entry->cm_last_need_notify_check) == 0)) {
 			/* Kick off a notification. */
-			entry->cm_state = CM_NEED_TO_NOTIFY;
+			entry->cm_state = CM_NEED_TO_NOTIFY_VALIDITY;
 			*when = cm_time_now;
 		} else
 		if (entry->cm_autorenew &&
@@ -962,10 +1117,11 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 		}
 		break;
 
-	case CM_NEED_TO_NOTIFY:
-		state->cm_notify_state = cm_notify_start(entry);
+	case CM_NEED_TO_NOTIFY_VALIDITY:
+		state->cm_notify_state = cm_notify_start(entry,
+							 cm_notify_event_validity_ending);
 		if (state->cm_notify_state != NULL) {
-			entry->cm_state = CM_NOTIFYING;
+			entry->cm_state = CM_NOTIFYING_VALIDITY;
 			/* Wait for status update, or poll. */
 			*readfd = cm_notify_get_fd(entry,
 						   state->cm_notify_state);
@@ -980,7 +1136,7 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 		}
 		break;
 
-	case CM_NOTIFYING:
+	case CM_NOTIFYING_VALIDITY:
 		if (cm_notify_ready(entry, state->cm_notify_state) == 0) {
 			cm_notify_done(entry, state->cm_notify_state);
 			state->cm_notify_state = NULL;
@@ -1011,7 +1167,169 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 		}
 		break;
 
+	case CM_NEED_TO_NOTIFY_REJECTION:
+		state->cm_notify_state = cm_notify_start(entry,
+							 cm_notify_event_rejected);
+		if (state->cm_notify_state != NULL) {
+			entry->cm_state = CM_NOTIFYING_REJECTION;
+			/* Wait for status update, or poll. */
+			*readfd = cm_notify_get_fd(entry,
+						   state->cm_notify_state);
+			if (*readfd == -1) {
+				*when = cm_time_soon;
+			} else {
+				*when = cm_time_no_time;
+			}
+		} else {
+			/* Failed to start notifying; try again. */
+			*when = cm_time_soonish;
+		}
+		break;
+
+	case CM_NOTIFYING_REJECTION:
+		if (cm_notify_ready(entry, state->cm_notify_state) == 0) {
+			cm_notify_done(entry, state->cm_notify_state);
+			state->cm_notify_state = NULL;
+			entry->cm_state = CM_CA_REJECTED;
+			*when = cm_time_soon;
+		} else {
+			/* Wait for status update, or poll. */
+			*readfd = cm_notify_get_fd(entry,
+						   state->cm_notify_state);
+			if (*readfd == -1) {
+				*when = cm_time_soon;
+			} else {
+				*when = cm_time_no_time;
+			}
+		}
+		break;
+
+	case CM_NEED_TO_NOTIFY_ISSUED_FAILED:
+		/* We should already have the lock here.  In cases where we're
+		 * resuming things at startup, try to acquire it if we don't
+		 * have it. */
+		if (!cm_writing_has_lock(entry) && !cm_writing_lock(entry)) {
+			/* Just hang out in this state while we're messing
+			 * around with the outside world for another entry. */
+			cm_log(3, "%s('%s') waiting for saving lock\n",
+			       entry->cm_busname, entry->cm_nickname);
+			*when = cm_time_soon;
+			break;
+		}
+		if (!cm_writing_unlock(entry)) {
+			/* If for some reason we fail to release the lock that
+			 * we have, try to release it again soon. */
+			*when = cm_time_soon;
+			cm_log(1, "%s('%s') failed to release saving "
+			       "lock, probably a bug\n",
+			       entry->cm_busname, entry->cm_nickname);
+			break;
+		}
+		state->cm_notify_state = cm_notify_start(entry,
+							 cm_notify_event_issued_not_saved);
+		if (state->cm_notify_state != NULL) {
+			entry->cm_state = CM_NOTIFYING_ISSUED_FAILED;
+			/* Wait for status update, or poll. */
+			*readfd = cm_notify_get_fd(entry,
+						   state->cm_notify_state);
+			if (*readfd == -1) {
+				*when = cm_time_soon;
+			} else {
+				*when = cm_time_no_time;
+			}
+		} else {
+			/* Failed to start notifying; try again. */
+			*when = cm_time_soonish;
+		}
+		break;
+
+	case CM_NOTIFYING_ISSUED_FAILED:
+		if (cm_notify_ready(entry, state->cm_notify_state) == 0) {
+			cm_notify_done(entry, state->cm_notify_state);
+			state->cm_notify_state = NULL;
+			entry->cm_state = CM_NEED_TO_SAVE_CERT;
+			*when = cm_time_soonish;
+		} else {
+			/* Wait for status update, or poll. */
+			*readfd = cm_notify_get_fd(entry,
+						   state->cm_notify_state);
+			if (*readfd == -1) {
+				*when = cm_time_soon;
+			} else {
+				*when = cm_time_no_time;
+			}
+		}
+		break;
+
+	case CM_NEED_TO_NOTIFY_ISSUED_SAVED:
+		/* We should already have the lock here.  In cases where we're
+		 * resuming things at startup, try to acquire it if we don't
+		 * have it. */
+		if (!cm_writing_has_lock(entry) && !cm_writing_lock(entry)) {
+			/* Just hang out in this state while we're messing
+			 * around with the outside world for another entry. */
+			cm_log(3, "%s('%s') waiting for saving lock\n",
+			       entry->cm_busname, entry->cm_nickname);
+			*when = cm_time_soon;
+			break;
+		}
+		if (!cm_writing_unlock(entry)) {
+			/* If for some reason we fail to release the lock that
+			 * we have, try to release it again soon. */
+			*when = cm_time_soon;
+			cm_log(1, "%s('%s') failed to release saving "
+			       "lock, probably a bug\n",
+			       entry->cm_busname, entry->cm_nickname);
+			break;
+		}
+		state->cm_notify_state = cm_notify_start(entry,
+							 cm_notify_event_issued_and_saved);
+		if (state->cm_notify_state != NULL) {
+			entry->cm_state = CM_NOTIFYING_ISSUED_SAVED;
+			/* Wait for status update, or poll. */
+			*readfd = cm_notify_get_fd(entry,
+						   state->cm_notify_state);
+			if (*readfd == -1) {
+				*when = cm_time_soon;
+			} else {
+				*when = cm_time_no_time;
+			}
+		} else {
+			/* Failed to start notifying; try again. */
+			*when = cm_time_soonish;
+		}
+		break;
+
+	case CM_NOTIFYING_ISSUED_SAVED:
+		if (cm_notify_ready(entry, state->cm_notify_state) == 0) {
+			cm_notify_done(entry, state->cm_notify_state);
+			state->cm_notify_state = NULL;
+			entry->cm_state = CM_MONITORING;
+			*when = cm_time_soon;
+		} else {
+			/* Wait for status update, or poll. */
+			*readfd = cm_notify_get_fd(entry,
+						   state->cm_notify_state);
+			if (*readfd == -1) {
+				*when = cm_time_soon;
+			} else {
+				*when = cm_time_no_time;
+			}
+		}
+		break;
+
+
 	case CM_NEWLY_ADDED:
+		/* Take the lock here because the database is opened read-write
+		 * in case we need to set a password on it. */
+		if (!cm_writing_lock(entry)) {
+			/* Just hang out in this state while we're messing
+			 * around with the outside world for another entry. */
+			cm_log(3, "%s('%s') waiting for reading lock\n",
+			       entry->cm_busname, entry->cm_nickname);
+			*when = cm_time_soon;
+			break;
+		}
 		/* We need to do some recon, and then decide what we need to
 		 * do to make things the way the user has specified that they
 		 * should be. */
@@ -1056,12 +1374,34 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 			} else
 			if (cm_keyiread_need_token(entry,
 						   state->cm_keyiread_state) == 0) {
+				if (!cm_writing_unlock(entry)) {
+					/* If for some reason we fail to
+					 * release the lock that we have, try
+					 * to release it again soon. */
+					*when = cm_time_soon;
+					cm_log(1, "%s('%s') failed to release "
+					       "reading lock, probably a bug\n",
+					       entry->cm_busname,
+					       entry->cm_nickname);
+					break;
+				}
 				/* If we need the token, just hang on. */
 				entry->cm_state = CM_NEWLY_ADDED_NEED_KEYINFO_READ_TOKEN;
 				*when = cm_time_now;
 			} else
 			if (cm_keyiread_need_pin(entry,
 						 state->cm_keyiread_state) == 0) {
+				if (!cm_writing_unlock(entry)) {
+					/* If for some reason we fail to
+					 * release the lock that we have, try
+					 * to release it again soon. */
+					*when = cm_time_soon;
+					cm_log(1, "%s('%s') failed to release "
+					       "reading lock, probably a bug\n",
+					       entry->cm_busname,
+					       entry->cm_nickname);
+					break;
+				}
 				/* If we need the PIN, just hang on. */
 				entry->cm_state = CM_NEWLY_ADDED_NEED_KEYINFO_READ_PIN;
 				*when = cm_time_now;
@@ -1136,6 +1476,15 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 		break;
 
 	case CM_NEWLY_ADDED_DECIDING:
+		if (!cm_writing_unlock(entry)) {
+			/* If for some reason we fail to release the lock that
+			 * we have, try to release it again soon. */
+			*when = cm_time_soon;
+			cm_log(1, "%s('%s') failed to release reading lock, "
+			       "probably a bug\n",
+			       entry->cm_busname, entry->cm_nickname);
+			break;
+		}
 		/* Decide what to do next.  Assign a CA if it doesn't have one
 		 * assigned to it already. */
 		if ((entry->cm_ca_nickname == NULL) &&
@@ -1178,6 +1527,9 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 		 * and EKU values, then try to pull them from the certificate,
 		 * too. */
 		if (entry->cm_cert != NULL) {
+			cm_store_set_if_not_set_s(entry,
+						  &entry->cm_template_subject_der,
+						  entry->cm_cert_subject_der);
 			cm_store_set_if_not_set_s(entry,
 						  &entry->cm_template_subject,
 						  entry->cm_cert_subject);
@@ -1196,6 +1548,12 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 			cm_store_set_if_not_set_s(entry,
 						  &entry->cm_template_eku,
 						  entry->cm_cert_eku);
+			cm_store_set_if_not_set_s(entry,
+						  &entry->cm_template_ns_comment,
+						  entry->cm_cert_ns_comment);
+			cm_store_set_if_not_set_s(entry,
+						  &entry->cm_template_profile,
+						  entry->cm_cert_profile);
 			cm_log(3, "%s('%s') has a certificate, monitoring it\n",
 			       entry->cm_busname, entry->cm_nickname);
 			entry->cm_state = CM_MONITORING;
@@ -1287,5 +1645,8 @@ cm_iterate_done(struct cm_store_entry *entry, void *cm_iterate_state)
 	cm_log(3, "%s('%s') ends in state '%s'\n",
 	       entry->cm_busname, entry->cm_nickname,
 	       cm_store_state_as_string(entry->cm_state));
+	if (cm_writing_has_lock(entry)) {
+		cm_writing_unlock(entry);
+	}
 	return 0;
 }
