@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009,2010,2011 Red Hat, Inc.
+ * Copyright (C) 2009,2010,2011,2013 Red Hat, Inc.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -69,6 +69,15 @@ cm_keygen_saved_keypair(struct cm_store_entry *entry,
 	return pvt->saved_keypair(entry, state);
 }
 
+/* Tell us if we need filesystem permissions to write the key. */
+int
+cm_keygen_need_perms(struct cm_store_entry *entry,
+		     struct cm_keygen_state *state)
+{
+	struct cm_keygen_state_pvt *pvt = (struct cm_keygen_state_pvt *) state;
+	return pvt->need_perms(entry, state);
+}
+
 /* Tell us if we need a PIN (or a new PIN) to access the key store. */
 int
 cm_keygen_need_pin(struct cm_store_entry *entry,
