@@ -61,22 +61,22 @@ cm_keyiread_o_main(int fd, struct cm_store_ca *ca, struct cm_store_entry *entry,
 	const char *alg;
 	int bits, length;
 	long error;
-	char *pin, *pubkey;
+	char *pin, *pubkey, *pubikey;
 	unsigned char *tmp;
 
 	util_o_init();
 	ERR_load_crypto_strings();
-	status = CM_STATUS_ERROR_INTERNAL;
+	status = CM_SUB_STATUS_INTERNAL_ERROR;
 	fp = fdopen(fd, "w");
 	if (fp == NULL) {
 		cm_log(1, "Unable to initialize I/O.\n");
-		_exit(CM_STATUS_ERROR_INTERNAL);
+		_exit(CM_SUB_STATUS_INTERNAL_ERROR);
 	}
 	pem = fopen(entry->cm_key_storage_location, "r");
 	if (pem != NULL) {
 		if (cm_pin_read_for_key(entry, &pin) != 0) {
 			cm_log(1, "Error reading key encryption PIN.\n");
-			_exit(CM_STATUS_ERROR_AUTH);
+			_exit(CM_SUB_STATUS_ERROR_AUTH);
 		}
 		memset(&cb_data, 0, sizeof(cb_data));
 		cb_data.entry = entry;
@@ -87,7 +87,7 @@ cm_keyiread_o_main(int fd, struct cm_store_ca *ca, struct cm_store_entry *entry,
 		if (pkey == NULL) {
 			cm_log(1, "Internal error reading key from \"%s\".\n",
 			       entry->cm_key_storage_location);
-			status = CM_STATUS_ERROR_AUTH; /* XXX */
+			status = CM_SUB_STATUS_ERROR_AUTH; /* XXX */
 		} else {
 			if ((pin != NULL) &&
 			    (strlen(pin) > 0) &&
@@ -96,7 +96,7 @@ cm_keyiread_o_main(int fd, struct cm_store_ca *ca, struct cm_store_entry *entry,
 				       "key '%s', though one was provided. "
 				       "Treating this as an error.\n",
 				       entry->cm_key_storage_location);
-				status = CM_STATUS_ERROR_AUTH; /* XXX */
+				status = CM_SUB_STATUS_ERROR_AUTH; /* XXX */
 			} else {
 				status = 0;
 			}
@@ -115,16 +115,25 @@ cm_keyiread_o_main(int fd, struct cm_store_ca *ca, struct cm_store_entry *entry,
 		alg = "";
 		bits = 0;
 		pubkey = "";
+		pubikey = "";
 		if (pkey != NULL) {
 			switch (EVP_PKEY_type(pkey->type)) {
 			case EVP_PKEY_RSA:
 				cm_log(3, "Key is an RSA key.\n");
 				alg = "RSA";
 				break;
+#ifdef CM_ENABLE_DSA
 			case EVP_PKEY_DSA:
 				cm_log(3, "Key is a DSA key.\n");
 				alg = "DSA";
 				break;
+#endif
+#ifdef CM_ENABLE_EC
+			case EVP_PKEY_EC:
+				cm_log(3, "Key is an EC key.\n");
+				alg = "EC";
+				break;
+#endif
 			default:
 				cm_log(3, "Key is for an unknown algorithm.\n");
 				alg = "";
@@ -133,12 +142,17 @@ cm_keyiread_o_main(int fd, struct cm_store_ca *ca, struct cm_store_entry *entry,
 			bits = EVP_PKEY_bits(pkey);
 			cm_log(3, "Key size is %d.\n", bits);
 			tmp = NULL;
+			length = i2d_PUBKEY(pkey, (unsigned char **) &tmp);
+			if (length > 0) {
+				pubikey = cm_store_hex_from_bin(NULL, tmp, length);
+			}
+			tmp = NULL;
 			length = i2d_PublicKey(pkey, (unsigned char **) &tmp);
 			if (length > 0) {
 				pubkey = cm_store_hex_from_bin(NULL, tmp, length);
 			}
 		}
-		fprintf(fp, "%s/%d/%s\n", alg, bits, pubkey);
+		fprintf(fp, "%s/%d/%s/%s\n", alg, bits, pubikey, pubkey);
 		status = 0;
 	} else {
 		while ((error = ERR_get_error()) != 0) {
@@ -174,7 +188,7 @@ cm_keyiread_o_need_pin(struct cm_store_entry *entry,
 	int status;
 	status = cm_subproc_get_exitstatus(entry, state->subproc);
 	if (WIFEXITED(status) &&
-	    (WEXITSTATUS(status) == CM_STATUS_ERROR_AUTH)) {
+	    (WEXITSTATUS(status) == CM_SUB_STATUS_ERROR_AUTH)) {
 		return 0;
 	}
 	return -1;
@@ -188,7 +202,7 @@ cm_keyiread_o_need_token(struct cm_store_entry *entry,
 	int status;
 	status = cm_subproc_get_exitstatus(entry, state->subproc);
 	if (WIFEXITED(status) &&
-	    (WEXITSTATUS(status) == CM_STATUS_ERROR_NO_TOKEN)) {
+	    (WEXITSTATUS(status) == CM_SUB_STATUS_ERROR_NO_TOKEN)) {
 		return 0;
 	}
 	return -1;
