@@ -127,8 +127,24 @@ cm_keyiread_read_data_from_buffer(struct cm_store_entry *entry, const char *p)
 	/* Break out the algorithm. */
 	q = p + strcspn(p, "/\r\n");
 	if (((q - p) == strlen("RSA")) &&
-	    (strncasecmp(p, "RSA", 3) == 0)) {
+	     (strncasecmp(p, "RSA", 3) == 0)) {
 		alg = cm_key_rsa;
+#ifdef CM_ENABLE_DSA
+	} else
+	if (((q - p) == strlen("DSA")) &&
+	    (strncasecmp(p, "DSA", 3) == 0)) {
+		alg = cm_key_dsa;
+#endif
+#ifdef CM_ENABLE_EC
+	} else
+	if (((q - p) == strlen("EC")) &&
+	    (strncasecmp(p, "EC", 2) == 0)) {
+		alg = cm_key_ecdsa;
+#endif
+	} else {
+		alg = cm_key_unspecified;
+	}
+	if (alg != cm_key_unspecified) {
 		p = q + strspn(q, "/\r\n");
 		q = p + strcspn(p, "/\r\n");
 		if (p != q) {
@@ -139,17 +155,24 @@ cm_keyiread_read_data_from_buffer(struct cm_store_entry *entry, const char *p)
 			}
 			p = q + strspn(q, "/\r\n");
 			q = p + strcspn(p, "/\r\n");
+			if (p != q) {
+				talloc_free(entry->cm_key_pubkey_info);
+				entry->cm_key_pubkey_info = talloc_strndup(entry,
+									   p, q - p);
+			}
+			p = q + strspn(q, "/\r\n");
+			q = p + strcspn(p, "/\r\n");
 			if (p != q) {
 				talloc_free(entry->cm_key_pubkey);
 				entry->cm_key_pubkey = talloc_strndup(entry,
 								      p, q - p);
-				p = q + strspn(q, "/\r\n");
-				q = p + strcspn(p, "/\r\n");
-				if (p != q) {
-					talloc_free(entry->cm_key_token);
-					entry->cm_key_token = talloc_strndup(entry,
-									     p, q - p);
-				}
+			}
+			p = q + strspn(q, "/\r\n");
+			q = p + strcspn(p, "/\r\n");
+			if (p != q) {
+				talloc_free(entry->cm_key_token);
+				entry->cm_key_token = talloc_strndup(entry,
+								     p, q - p);
 			}
 		}
 	}
