@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009,2010,2011,2012 Red Hat, Inc.
+ * Copyright (C) 2009,2010,2011,2012,2013,2014 Red Hat, Inc.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,6 +36,7 @@
 #include "env.h"
 #include "store.h"
 #include "store-int.h"
+#include "submit-e.h"
 #include "log.h"
 #include "tm.h"
 
@@ -57,14 +58,17 @@ enum cm_store_file_field {
 	cm_store_entry_field_key_pin,
 	cm_store_entry_field_key_pin_file,
 	cm_store_entry_field_key_pubkey,
+	cm_store_entry_field_key_pubkey_info,
 
 	cm_store_entry_field_cert_storage_type,
 	cm_store_entry_field_cert_storage_location,
 	cm_store_entry_field_cert_token,
 	cm_store_entry_field_cert_nickname,
 
+	cm_store_entry_field_cert_issuer_der,
 	cm_store_entry_field_cert_issuer,
 	cm_store_entry_field_cert_serial,
+	cm_store_entry_field_cert_subject_der,
 	cm_store_entry_field_cert_subject,
 	cm_store_entry_field_cert_spki,
 	cm_store_entry_field_cert_not_before,
@@ -74,17 +78,30 @@ enum cm_store_file_field {
 	cm_store_entry_field_cert_principal,
 	cm_store_entry_field_cert_ku,
 	cm_store_entry_field_cert_eku,
+	cm_store_entry_field_cert_is_ca,
+	cm_store_entry_field_cert_ca_path_length,
+	cm_store_entry_field_cert_crl_distribution_point,
+	cm_store_entry_field_cert_ocsp_location,
+	cm_store_entry_field_cert_ns_comment,
+	cm_store_entry_field_cert_profile,
 
 	cm_store_entry_field_last_expiration_check,
 	cm_store_entry_field_last_need_notify_check,
 	cm_store_entry_field_last_need_enroll_check,
 
+	cm_store_entry_field_template_subject_der,
 	cm_store_entry_field_template_subject,
 	cm_store_entry_field_template_hostname,
 	cm_store_entry_field_template_email,
 	cm_store_entry_field_template_principal,
 	cm_store_entry_field_template_ku,
 	cm_store_entry_field_template_eku,
+	cm_store_entry_field_template_is_ca,
+	cm_store_entry_field_template_ca_path_length,
+	cm_store_entry_field_template_crl_distribution_point,
+	cm_store_entry_field_template_ocsp_location,
+	cm_store_entry_field_template_ns_comment,
+	cm_store_entry_field_template_profile,
 
 	cm_store_entry_field_challenge_password,
 
@@ -137,14 +154,17 @@ static struct cm_store_file_field_list {
 	{cm_store_entry_field_key_pin, "key_pin"},
 	{cm_store_entry_field_key_pin_file, "key_pin_file"},
 	{cm_store_entry_field_key_pubkey, "key_pubkey"},
+	{cm_store_entry_field_key_pubkey_info, "key_pubkey_info"},
 
 	{cm_store_entry_field_cert_storage_type, "cert_storage_type"},
 	{cm_store_entry_field_cert_storage_location, "cert_storage_location"},
 	{cm_store_entry_field_cert_token, "cert_token"},
 	{cm_store_entry_field_cert_nickname, "cert_nickname"},
 
+	{cm_store_entry_field_cert_issuer_der, "cert_issuer_der"},
 	{cm_store_entry_field_cert_issuer, "cert_issuer"},
 	{cm_store_entry_field_cert_serial, "cert_serial"},
+	{cm_store_entry_field_cert_subject_der, "cert_subject_der"},
 	{cm_store_entry_field_cert_subject, "cert_subject"},
 	{cm_store_entry_field_cert_spki, "cert_spki"},
 	{cm_store_entry_field_cert_not_before, "cert_not_before"}, /* right */
@@ -156,17 +176,31 @@ static struct cm_store_file_field_list {
 	{cm_store_entry_field_cert_principal, "cert_principal"},
 	{cm_store_entry_field_cert_ku, "cert_ku"},
 	{cm_store_entry_field_cert_eku, "cert_eku"},
+	{cm_store_entry_field_cert_is_ca, "cert_is_ca"},
+	{cm_store_entry_field_cert_ca_path_length, "cert_ca_path_length"},
+	{cm_store_entry_field_cert_crl_distribution_point, "cert_crldp"},
+	{cm_store_entry_field_cert_ocsp_location, "cert_ocsp"},
+	{cm_store_entry_field_cert_ns_comment, "cert_ns_comment"},
+	{cm_store_entry_field_cert_profile, "cert_profile"},
 
 	{cm_store_entry_field_last_expiration_check, "last_expiration_check"},
 	{cm_store_entry_field_last_need_notify_check, "last_need_notify_check"},
 	{cm_store_entry_field_last_need_enroll_check, "last_need_enroll_check"},
 
+	{cm_store_entry_field_template_subject_der, "template_subject_der"},
 	{cm_store_entry_field_template_subject, "template_subject"},
 	{cm_store_entry_field_template_hostname, "template_hostname"},
 	{cm_store_entry_field_template_email, "template_email"},
 	{cm_store_entry_field_template_principal, "template_principal"},
 	{cm_store_entry_field_template_ku, "template_ku"},
 	{cm_store_entry_field_template_eku, "template_eku"},
+	{cm_store_entry_field_template_is_ca, "template_is_ca"},
+	{cm_store_entry_field_template_ca_path_length, "template_ca_path_length"},
+	{cm_store_entry_field_template_crl_distribution_point, "template_crldp"},
+	{cm_store_entry_field_template_ocsp_location, "template_ocsp"},
+	{cm_store_entry_field_template_ns_comment, "template_ns_comment"},
+	{cm_store_entry_field_template_profile, "template_profile"}, /* right */
+	{cm_store_entry_field_template_profile, "ca_profile"}, /* wrong */
 
 	{cm_store_entry_field_challenge_password, "challenge_password"},
 
@@ -259,13 +293,17 @@ cm_store_should_ignore_file(const char *filename)
 static char **
 cm_store_file_read_lines(void *parent, FILE *fp)
 {
-	char buf[LINE_MAX], *s, *t, **lines, **tlines;
+	char *buf, *s, *t, **lines, **tlines;
 	int n_lines, trim, offset;
+	size_t buflen;
+
 	s = NULL;
 	lines = NULL;
 	n_lines = 0;
 	trim = 1;
-	while (fgets(buf, sizeof(buf), fp) == buf) {
+	buf = NULL;
+	buflen = 0;
+	while (getline(&buf, &buflen, fp) > 0) {
 		offset = 0;
 		switch (buf[0]) {
 		case '=':
@@ -307,6 +345,7 @@ cm_store_file_read_lines(void *parent, FILE *fp)
 			break;
 		}
 	}
+	free(buf);
 	/* If we were reading a line, append it to the list. */
 	if (s != NULL) {
 		tlines = talloc_realloc(parent, lines, char *, n_lines + 2);
@@ -394,6 +433,7 @@ cm_store_entry_read(void *parent, const char *filename, FILE *fp)
 		s = cm_store_file_read_lines(ret, fp);
 		ret->cm_busname = cm_store_entry_next_busname(ret);
 		ret->cm_store_private = talloc_strdup(ret, filename);
+		ret->cm_template_ca_path_length = -1;
 		for (i = 0; (s != NULL) && (s[i] != NULL); i++) {
 			p = s[i];
 			field = cm_store_file_field_of_line(p);
@@ -415,6 +455,19 @@ cm_store_entry_read(void *parent, const char *filename, FILE *fp)
 				if (strcasecmp(s[i], "RSA") == 0) {
 					ret->cm_key_type.cm_key_algorithm =
 						cm_key_rsa;
+#ifdef CM_ENABLE_DSA
+				} else
+				if (strcasecmp(s[i], "DSA") == 0) {
+					ret->cm_key_type.cm_key_algorithm =
+						cm_key_dsa;
+#endif
+#ifdef CM_ENABLE_EC
+				} else
+				if ((strcasecmp(s[i], "ECDSA") == 0) ||
+				    (strcasecmp(s[i], "EC") == 0)) {
+					ret->cm_key_type.cm_key_algorithm =
+						cm_key_ecdsa;
+#endif
 				} else {
 					ret->cm_key_type.cm_key_algorithm =
 						cm_key_unspecified;
@@ -425,6 +478,19 @@ cm_store_entry_read(void *parent, const char *filename, FILE *fp)
 				if (strcasecmp(s[i], "RSA") == 0) {
 					ret->cm_key_type.cm_key_gen_algorithm =
 						cm_key_rsa;
+#ifdef CM_ENABLE_DSA
+				} else
+				if (strcasecmp(s[i], "DSA") == 0) {
+					ret->cm_key_type.cm_key_gen_algorithm =
+						cm_key_dsa;
+#endif
+#ifdef CM_ENABLE_EC
+				} else
+				if ((strcasecmp(s[i], "ECDSA") == 0) ||
+				    (strcasecmp(s[i], "EC") == 0)) {
+					ret->cm_key_type.cm_key_gen_algorithm =
+						cm_key_ecdsa;
+#endif
 				} else {
 					ret->cm_key_type.cm_key_gen_algorithm =
 						cm_key_unspecified;
@@ -487,6 +553,9 @@ cm_store_entry_read(void *parent, const char *filename, FILE *fp)
 			case cm_store_entry_field_key_pubkey:
 				ret->cm_key_pubkey = free_if_empty(p);
 				break;
+			case cm_store_entry_field_key_pubkey_info:
+				ret->cm_key_pubkey_info = free_if_empty(p);
+				break;
 			case cm_store_entry_field_cert_storage_type:
 				if (strcasecmp(p, "FILE") == 0) {
 					ret->cm_cert_storage_type =
@@ -516,12 +585,18 @@ cm_store_entry_read(void *parent, const char *filename, FILE *fp)
 			case cm_store_entry_field_cert_nickname:
 				ret->cm_cert_nickname = free_if_empty(p);
 				break;
+			case cm_store_entry_field_cert_issuer_der:
+				ret->cm_cert_issuer_der = free_if_empty(p);
+				break;
 			case cm_store_entry_field_cert_issuer:
 				ret->cm_cert_issuer = free_if_empty(p);
 				break;
 			case cm_store_entry_field_cert_serial:
 				ret->cm_cert_serial = free_if_empty(p);
 				break;
+			case cm_store_entry_field_cert_subject_der:
+				ret->cm_cert_subject_der = free_if_empty(p);
+				break;
 			case cm_store_entry_field_cert_subject:
 				ret->cm_cert_subject = free_if_empty(p);
 				break;
@@ -556,6 +631,28 @@ cm_store_entry_read(void *parent, const char *filename, FILE *fp)
 			case cm_store_entry_field_cert_eku:
 				ret->cm_cert_eku = free_if_empty(p);
 				break;
+			case cm_store_entry_field_cert_is_ca:
+				ret->cm_cert_is_ca = atoi(p);
+				talloc_free(p);
+				break;
+			case cm_store_entry_field_cert_ca_path_length:
+				ret->cm_cert_ca_path_length = atoi(p);
+				talloc_free(p);
+				break;
+			case cm_store_entry_field_cert_crl_distribution_point:
+				ret->cm_cert_crl_distribution_point =
+					free_if_empty_multi(ret, p);
+				break;
+			case cm_store_entry_field_cert_ocsp_location:
+				ret->cm_cert_ocsp_location =
+					free_if_empty_multi(ret, p);
+				break;
+			case cm_store_entry_field_cert_ns_comment:
+				ret->cm_cert_ns_comment = free_if_empty(p);
+				break;
+			case cm_store_entry_field_cert_profile:
+				ret->cm_cert_profile = free_if_empty(p);
+				break;
 			case cm_store_entry_field_last_expiration_check:
 				/* backward compatibility before we split them
 				 * into two settings */
@@ -575,6 +672,9 @@ cm_store_entry_read(void *parent, const char *filename, FILE *fp)
 					cm_store_time_from_timestamp(p);
 				talloc_free(p);
 				break;
+			case cm_store_entry_field_template_subject_der:
+				ret->cm_template_subject_der = free_if_empty(p);
+				break;
 			case cm_store_entry_field_template_subject:
 				ret->cm_template_subject = free_if_empty(p);
 				break;
@@ -596,6 +696,28 @@ cm_store_entry_read(void *parent, const char *filename, FILE *fp)
 			case cm_store_entry_field_template_eku:
 				ret->cm_template_eku = free_if_empty(p);
 				break;
+			case cm_store_entry_field_template_is_ca:
+				ret->cm_template_is_ca = atoi(p);
+				talloc_free(p);
+				break;
+			case cm_store_entry_field_template_ca_path_length:
+				ret->cm_template_ca_path_length = atoi(p);
+				talloc_free(p);
+				break;
+			case cm_store_entry_field_template_crl_distribution_point:
+				ret->cm_template_crl_distribution_point =
+					free_if_empty_multi(ret, p);
+				break;
+			case cm_store_entry_field_template_ocsp_location:
+				ret->cm_template_ocsp_location =
+					free_if_empty_multi(ret, p);
+				break;
+			case cm_store_entry_field_template_ns_comment:
+				ret->cm_template_ns_comment = free_if_empty(p);
+				break;
+			case cm_store_entry_field_template_profile:
+				ret->cm_template_profile = free_if_empty(p);
+				break;
 			case cm_store_entry_field_challenge_password:
 				ret->cm_challenge_password = free_if_empty(p);
 				break;
@@ -708,12 +830,15 @@ cm_store_ca_read(void *parent, const char *filename, FILE *fp)
 			case cm_store_entry_field_key_pin:
 			case cm_store_entry_field_key_pin_file:
 			case cm_store_entry_field_key_pubkey:
+			case cm_store_entry_field_key_pubkey_info:
 			case cm_store_entry_field_cert_storage_type:
 			case cm_store_entry_field_cert_storage_location:
 			case cm_store_entry_field_cert_token:
 			case cm_store_entry_field_cert_nickname:
+			case cm_store_entry_field_cert_issuer_der:
 			case cm_store_entry_field_cert_issuer:
 			case cm_store_entry_field_cert_serial:
+			case cm_store_entry_field_cert_subject_der:
 			case cm_store_entry_field_cert_subject:
 			case cm_store_entry_field_cert_spki:
 			case cm_store_entry_field_cert_not_before:
@@ -723,15 +848,28 @@ cm_store_ca_read(void *parent, const char *filename, FILE *fp)
 			case cm_store_entry_field_cert_principal:
 			case cm_store_entry_field_cert_ku:
 			case cm_store_entry_field_cert_eku:
+			case cm_store_entry_field_cert_is_ca:
+			case cm_store_entry_field_cert_ca_path_length:
+			case cm_store_entry_field_cert_crl_distribution_point:
+			case cm_store_entry_field_cert_ocsp_location:
+			case cm_store_entry_field_cert_ns_comment:
+			case cm_store_entry_field_cert_profile:
 			case cm_store_entry_field_last_expiration_check:
 			case cm_store_entry_field_last_need_notify_check:
 			case cm_store_entry_field_last_need_enroll_check:
+			case cm_store_entry_field_template_subject_der:
 			case cm_store_entry_field_template_subject:
 			case cm_store_entry_field_template_hostname:
 			case cm_store_entry_field_template_email:
 			case cm_store_entry_field_template_principal:
 			case cm_store_entry_field_template_ku:
 			case cm_store_entry_field_template_eku:
+			case cm_store_entry_field_template_is_ca:
+			case cm_store_entry_field_template_ca_path_length:
+			case cm_store_entry_field_template_crl_distribution_point:
+			case cm_store_entry_field_template_ocsp_location:
+			case cm_store_entry_field_template_ns_comment:
+			case cm_store_entry_field_template_profile:
 			case cm_store_entry_field_challenge_password:
 			case cm_store_entry_field_csr:
 			case cm_store_entry_field_spkac:
@@ -895,6 +1033,18 @@ cm_store_entry_write(FILE *fp, struct cm_store_entry *entry)
 		cm_store_file_write_str(fp, cm_store_entry_field_key_type,
 					"RSA");
 		break;
+#ifdef CM_ENABLE_DSA
+	case cm_key_dsa:
+		cm_store_file_write_str(fp, cm_store_entry_field_key_type,
+					"DSA");
+		break;
+#endif
+#ifdef CM_ENABLE_EC
+	case cm_key_ecdsa:
+		cm_store_file_write_str(fp, cm_store_entry_field_key_type,
+					"EC");
+		break;
+#endif
 	}
 	switch (entry->cm_key_type.cm_key_gen_algorithm) {
 	case cm_key_unspecified:
@@ -905,6 +1055,18 @@ cm_store_entry_write(FILE *fp, struct cm_store_entry *entry)
 		cm_store_file_write_str(fp, cm_store_entry_field_key_gen_type,
 					"RSA");
 		break;
+#ifdef CM_ENABLE_DSA
+	case cm_key_dsa:
+		cm_store_file_write_str(fp, cm_store_entry_field_key_gen_type,
+					"DSA");
+		break;
+#endif
+#ifdef CM_ENABLE_EC
+	case cm_key_ecdsa:
+		cm_store_file_write_str(fp, cm_store_entry_field_key_gen_type,
+					"EC");
+		break;
+#endif
 	}
 	cm_store_file_write_int(fp, cm_store_entry_field_key_size,
 				entry->cm_key_type.cm_key_size);
@@ -942,6 +1104,8 @@ cm_store_entry_write(FILE *fp, struct cm_store_entry *entry)
 				entry->cm_key_pin_file);
 	cm_store_file_write_str(fp, cm_store_entry_field_key_pubkey,
 				entry->cm_key_pubkey);
+	cm_store_file_write_str(fp, cm_store_entry_field_key_pubkey_info,
+				entry->cm_key_pubkey_info);
 
 	switch (entry->cm_cert_storage_type) {
 	case cm_cert_storage_file:
@@ -962,10 +1126,14 @@ cm_store_entry_write(FILE *fp, struct cm_store_entry *entry)
 	cm_store_file_write_str(fp, cm_store_entry_field_cert_nickname,
 				entry->cm_cert_nickname);
 
+	cm_store_file_write_str(fp, cm_store_entry_field_cert_issuer_der,
+				entry->cm_cert_issuer_der);
 	cm_store_file_write_str(fp, cm_store_entry_field_cert_issuer,
 				entry->cm_cert_issuer);
 	cm_store_file_write_str(fp, cm_store_entry_field_cert_serial,
 				entry->cm_cert_serial);
+	cm_store_file_write_str(fp, cm_store_entry_field_cert_subject_der,
+				entry->cm_cert_subject_der);
 	cm_store_file_write_str(fp, cm_store_entry_field_cert_subject,
 				entry->cm_cert_subject);
 	cm_store_file_write_str(fp, cm_store_entry_field_cert_spki,
@@ -990,6 +1158,18 @@ cm_store_entry_write(FILE *fp, struct cm_store_entry *entry)
 				entry->cm_cert_ku);
 	cm_store_file_write_str(fp, cm_store_entry_field_cert_eku,
 				entry->cm_cert_eku);
+	cm_store_file_write_int(fp, cm_store_entry_field_cert_is_ca,
+				entry->cm_cert_is_ca ? 1 : 0);
+	cm_store_file_write_int(fp, cm_store_entry_field_cert_ca_path_length,
+				entry->cm_cert_ca_path_length);
+	cm_store_file_write_strs(fp, cm_store_entry_field_cert_crl_distribution_point,
+				 entry->cm_cert_crl_distribution_point);
+	cm_store_file_write_strs(fp, cm_store_entry_field_cert_ocsp_location,
+				 entry->cm_cert_ocsp_location);
+	cm_store_file_write_str(fp, cm_store_entry_field_cert_ns_comment,
+				entry->cm_cert_ns_comment);
+	cm_store_file_write_str(fp, cm_store_entry_field_cert_profile,
+				entry->cm_cert_profile);
 
 	cm_store_file_write_str(fp, cm_store_entry_field_last_need_notify_check,
 				cm_store_timestamp_from_time(entry->cm_last_need_notify_check,
@@ -997,6 +1177,8 @@ cm_store_entry_write(FILE *fp, struct cm_store_entry *entry)
 	cm_store_file_write_str(fp, cm_store_entry_field_last_need_enroll_check,
 				cm_store_timestamp_from_time(entry->cm_last_need_enroll_check,
 							     timestamp));
+	cm_store_file_write_str(fp, cm_store_entry_field_template_subject_der,
+				entry->cm_template_subject_der);
 	cm_store_file_write_str(fp, cm_store_entry_field_template_subject,
 				entry->cm_template_subject);
 	cm_store_file_write_strs(fp, cm_store_entry_field_template_hostname,
@@ -1009,6 +1191,19 @@ cm_store_entry_write(FILE *fp, struct cm_store_entry *entry)
 				entry->cm_template_ku);
 	cm_store_file_write_str(fp, cm_store_entry_field_template_eku,
 				entry->cm_template_eku);
+	cm_store_file_write_int(fp, cm_store_entry_field_template_is_ca,
+				entry->cm_template_is_ca ? 1 : 0);
+	cm_store_file_write_int(fp, cm_store_entry_field_template_ca_path_length,
+				entry->cm_template_ca_path_length);
+	cm_store_file_write_strs(fp, cm_store_entry_field_template_crl_distribution_point,
+				 entry->cm_template_crl_distribution_point);
+	cm_store_file_write_strs(fp,
+				 cm_store_entry_field_template_ocsp_location,
+				 entry->cm_template_ocsp_location);
+	cm_store_file_write_str(fp, cm_store_entry_field_template_ns_comment,
+				entry->cm_template_ns_comment);
+	cm_store_file_write_str(fp, cm_store_entry_field_template_profile,
+				entry->cm_template_profile);
 
 	cm_store_file_write_str(fp, cm_store_entry_field_challenge_password,
 				entry->cm_challenge_password);
@@ -1064,7 +1259,7 @@ cm_store_entry_delete(struct cm_store_entry *entry)
 			talloc_free(entry->cm_store_private);
 			entry->cm_store_private = NULL;
 		} else {
-			cm_log(1, "Failed to remove file \"%s\": %s.\n",
+			cm_log(0, "Failed to remove file \"%s\": %s.\n",
 			       filename, strerror(errno));
 		}
 	} else {
@@ -1106,7 +1301,7 @@ cm_store_entry_save(struct cm_store_entry *entry)
 	FILE *fp;
 	char timestamp[15], path[PATH_MAX];
 	int i, fd = -1, give_up;
-	const char *directory;
+	const char *directory, *dest;
 
 	if (entry->cm_store_private == NULL) {
 		cm_store_timestamp_from_time(cm_time(NULL), timestamp);
@@ -1166,11 +1361,19 @@ cm_store_entry_save(struct cm_store_entry *entry)
 	if (fp != NULL) {
 		if (cm_store_entry_write(fp, entry) == 0) {
 			fclose(fp);
-			rename(path, (const char *) entry->cm_store_private);
+			dest = (const char *) entry->cm_store_private;
+			if (rename(path, dest) != 0) {
+				cm_log(0, "Error renaming \"%s\" to \"%s\": "
+				       "%s.\n", path, dest, strerror(errno));
+				return -1;
+			}
 			return 0;
 		} else {
 			fclose(fp);
-			remove(path);
+			if (remove(path) != 0) {
+				cm_log(0, "Error removing \"%s\": %s.\n", path,
+				       strerror(errno));
+			}
 			return -1;
 		}
 	} else {
@@ -1208,6 +1411,11 @@ cm_store_get_all_entries(void *parent)
 					ret[j] = cm_store_entry_read(ret,
 								     globs.gl_pathv[i],
 								     fp);
+					if ((ret[j] != NULL) &&
+					    (ret[j]->cm_nickname == NULL)) {
+						talloc_free(ret[j]);
+						ret[j] = NULL;
+					}
 					if (ret[j] != NULL) {
 						/* Check for duplicate names. */
 						for (k = 0; k < j; k++) {
@@ -1304,7 +1512,7 @@ cm_store_ca_save(struct cm_store_ca *ca)
 	FILE *fp;
 	char timestamp[15], path[PATH_MAX];
 	int i, fd = -1, give_up;
-	const char *directory;
+	const char *directory, *dest;
 
 	if (ca->cm_store_private == NULL) {
 		cm_store_timestamp_from_time(cm_time(NULL), timestamp);
@@ -1363,12 +1571,21 @@ cm_store_ca_save(struct cm_store_ca *ca)
 	if (fp != NULL) {
 		if (cm_store_ca_write(fp, ca) == 0) {
 			fclose(fp);
-			rename(path, (const char *) ca->cm_store_private);
+			dest = (const char *) ca->cm_store_private;
+			if (rename(path, dest) != 0) {
+				cm_log(0, "Error renaming \"%s\" to \"%s\": "
+				       "%s.\n", path, dest, strerror(errno));
+				return -1;
+			}
+			return 0;
 		} else {
 			fclose(fp);
-			remove(path);
+			if (remove(path) != 0) {
+				cm_log(0, "Error removing \"%s\": %s.\n", path,
+				       strerror(errno));
+			}
+			return -1;
 		}
-		return 0;
 	} else {
 		cm_log(1, "Error opening \"%s\" for writing: %s.\n", path,
 		       strerror(errno));
@@ -1394,7 +1611,7 @@ cm_store_get_all_cas(void *parent)
 	if (glob(path, 0, NULL, &globs) != 0) {
 		globs.gl_pathc = 0;
 	}
-	ret = talloc_array_ptrtype(parent, ret, globs.gl_pathc + 4);
+	ret = talloc_array_ptrtype(parent, ret, globs.gl_pathc + 5);
 	if (ret != NULL) {
 		for (i = 0, j = 0; i < globs.gl_pathc; i++) {
 			p = globs.gl_pathv[i];
@@ -1406,6 +1623,11 @@ cm_store_get_all_cas(void *parent)
 				ret[j] = cm_store_ca_read(ret,
 							  globs.gl_pathv[i],
 							  fp);
+				if ((ret[j] != NULL) &&
+				    (ret[j]->cm_nickname == NULL)) {
+					talloc_free(ret[j]);
+					ret[j] = NULL;
+				}
 				if (ret[j] != NULL) {
 					/* Check for duplicate names. */
 					for (k = 0; k < j; k++) {
@@ -1479,6 +1701,26 @@ cm_store_get_all_cas(void *parent)
 								      CM_CERTMASTER_HELPER_PATH);
 			j++;
 		}
+#endif
+#ifdef WITH_IPA
+		/* Make sure we get at least 1 dogtag-ipa-renew-agent entry. */
+		for (k = 0; k < j; k++) {
+			if ((ret[k]->cm_ca_type == cm_ca_external) &&
+			    (strcmp(ret[k]->cm_nickname,
+				    CM_DOGTAG_IPA_RENEW_AGENT_CA_NAME) == 0)) {
+				break;
+			}
+		}
+		if (k == j) {
+			ret[j] = cm_store_ca_new(ret);
+			ret[j]->cm_busname = cm_store_ca_next_busname(ret[j]);
+			ret[j]->cm_nickname = talloc_strdup(ret[j],
+							    CM_DOGTAG_IPA_RENEW_AGENT_CA_NAME);
+			ret[j]->cm_ca_type = cm_ca_external;
+			ret[j]->cm_ca_external_helper = talloc_strdup(ret[j],
+								      CM_DOGTAG_IPA_RENEW_AGENT_HELPER_PATH);
+			j++;
+		}
 #endif
 		ret[j] = NULL;
 	}
@@ -1511,14 +1753,17 @@ cm_store_entry_dup(void *parent, struct cm_store_entry *entry)
 		ret->cm_key_pin = NULL;
 	}
 	ret->cm_key_pubkey = cm_store_maybe_strdup(ret, entry->cm_key_pubkey);
+	ret->cm_key_pubkey_info = cm_store_maybe_strdup(ret, entry->cm_key_pubkey_info);
 
 	ret->cm_cert_storage_type = entry->cm_cert_storage_type;
 	ret->cm_cert_storage_location = cm_store_maybe_strdup(ret, entry->cm_cert_storage_location);
 	ret->cm_cert_token = cm_store_maybe_strdup(ret, entry->cm_cert_token);
 	ret->cm_cert_nickname = cm_store_maybe_strdup(ret, entry->cm_cert_nickname);
 
+	ret->cm_cert_issuer_der = cm_store_maybe_strdup(ret, entry->cm_cert_issuer_der);
 	ret->cm_cert_issuer = cm_store_maybe_strdup(ret, entry->cm_cert_issuer);
 	ret->cm_cert_serial = cm_store_maybe_strdup(ret, entry->cm_cert_serial);
+	ret->cm_cert_subject_der = cm_store_maybe_strdup(ret, entry->cm_cert_subject_der);
 	ret->cm_cert_subject = cm_store_maybe_strdup(ret, entry->cm_cert_subject);
 	ret->cm_cert_spki = cm_store_maybe_strdup(ret, entry->cm_cert_spki);
 	ret->cm_cert_not_before = entry->cm_cert_not_before;
@@ -1528,18 +1773,32 @@ cm_store_entry_dup(void *parent, struct cm_store_entry *entry)
 	ret->cm_cert_principal = cm_store_maybe_strdupv(ret, entry->cm_cert_principal);
 	ret->cm_cert_ku = cm_store_maybe_strdup(ret, entry->cm_cert_ku);
 	ret->cm_cert_eku = cm_store_maybe_strdup(ret, entry->cm_cert_eku);
+	ret->cm_cert_is_ca = entry->cm_cert_is_ca;
+	ret->cm_cert_ca_path_length = entry->cm_cert_ca_path_length;
+	ret->cm_cert_crl_distribution_point = cm_store_maybe_strdupv(ret, entry->cm_cert_crl_distribution_point);
+	ret->cm_cert_ocsp_location = cm_store_maybe_strdupv(ret, entry->cm_cert_ocsp_location);
+	ret->cm_cert_ns_comment = cm_store_maybe_strdup(ret, entry->cm_cert_ns_comment);
+	ret->cm_cert_profile = cm_store_maybe_strdup(ret,
+						     entry->cm_cert_profile);
 
 	ret->cm_last_need_notify_check = entry->cm_last_need_notify_check;
 	ret->cm_last_need_enroll_check = entry->cm_last_need_enroll_check;
 	ret->cm_notification_method = entry->cm_notification_method;
 	ret->cm_notification_destination = cm_store_maybe_strdup(ret, entry->cm_notification_destination);
 
+	ret->cm_template_subject_der = cm_store_maybe_strdup(ret, entry->cm_template_subject_der);
 	ret->cm_template_subject = cm_store_maybe_strdup(ret, entry->cm_template_subject);
 	ret->cm_template_hostname = cm_store_maybe_strdupv(ret, entry->cm_template_hostname);
 	ret->cm_template_email = cm_store_maybe_strdupv(ret, entry->cm_template_email);
 	ret->cm_template_principal = cm_store_maybe_strdupv(ret, entry->cm_template_principal);
 	ret->cm_template_ku = cm_store_maybe_strdup(ret, entry->cm_template_ku);
 	ret->cm_template_eku = cm_store_maybe_strdup(ret, entry->cm_template_eku);
+	ret->cm_template_is_ca = entry->cm_template_is_ca;
+	ret->cm_template_ca_path_length = entry->cm_template_ca_path_length;
+	ret->cm_template_crl_distribution_point = cm_store_maybe_strdupv(ret, entry->cm_template_crl_distribution_point);
+	ret->cm_template_ocsp_location = cm_store_maybe_strdupv(ret, entry->cm_template_ocsp_location);
+	ret->cm_template_ns_comment = cm_store_maybe_strdup(ret, entry->cm_template_ns_comment);
+	ret->cm_template_profile = cm_store_maybe_strdup(ret, entry->cm_template_profile);
 
 	ret->cm_challenge_password = cm_store_maybe_strdup(ret, entry->cm_challenge_password);
 	ret->cm_csr = cm_store_maybe_strdup(ret, entry->cm_csr);
