@@ -18,14 +18,20 @@
 #ifndef cmsubmite_h
 #define cmsubmite_h
 
+#define CM_DOGTAG_IPA_RENEW_AGENT_CA_NAME "dogtag-ipa-renew-agent"
+#define CM_DOGTAG_IPA_RENEW_AGENT_HELPER_PATH \
+	CM_DEFAULT_HELPER_PATH "/dogtag-ipa-renew-agent-submit"
+
 enum cm_external_status {
-	CM_STATUS_ISSUED = 0,
-	CM_STATUS_WAIT = 1,
-	CM_STATUS_REJECTED = 2,
-	CM_STATUS_UNREACHABLE = 3,
-	CM_STATUS_UNCONFIGURED = 4,
-	CM_STATUS_WAIT_WITH_DELAY = 5,
+	CM_SUBMIT_STATUS_ISSUED = 0,
+	CM_SUBMIT_STATUS_WAIT = 1,
+	CM_SUBMIT_STATUS_REJECTED = 2,
+	CM_SUBMIT_STATUS_UNREACHABLE = 3,
+	CM_SUBMIT_STATUS_UNCONFIGURED = 4,
+	CM_SUBMIT_STATUS_WAIT_WITH_DELAY = 5,
+	CM_SUBMIT_STATUS_OPERATION_NOT_SUPPORTED = 6,
 };
+const char *cm_submit_e_status_text(enum cm_external_status status);
 
 #define CM_SUBMIT_REQ_SUBJECT_ENV "CERTMONGER_REQ_SUBJECT"
 #define CM_SUBMIT_REQ_HOSTNAME_ENV "CERTMONGER_REQ_HOSTNAME"
@@ -33,6 +39,12 @@ enum cm_external_status {
 #define CM_SUBMIT_REQ_EMAIL_ENV "CERTMONGER_REQ_EMAIL"
 #define CM_SUBMIT_OPERATION_ENV "CERTMONGER_OPERATION"
 #define CM_SUBMIT_CSR_ENV "CERTMONGER_CSR"
+#define CM_SUBMIT_SPKAC_ENV "CERTMONGER_SPKAC"
+#define CM_SUBMIT_SPKI_ENV "CERTMONGER_SPKI"
+#define CM_SUBMIT_KEY_TYPE_ENV "CERTMONGER_KEY_TYPE"
 #define CM_SUBMIT_COOKIE_ENV "CERTMONGER_CA_COOKIE"
+#define CM_SUBMIT_CA_NICKNAME_ENV "CERTMONGER_CA_NICKNAME"
+#define CM_SUBMIT_PROFILE_ENV "CERTMONGER_CA_PROFILE"
+#define CM_SUBMIT_CERTIFICATE_ENV "CERTMONGER_CERTIFICATE"
 
 #endif
