@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009,2010,2011 Red Hat, Inc.
+ * Copyright (C) 2009,2010,2011,2012 Red Hat, Inc.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,10 +39,10 @@
 #include "log.h"
 #include "submit-u.h"
 
-#define BASE64_ALPHABET "0123456789" \
-			"ABCDEFGHIJKLMNOPQRSTUVWXYZ" \
+#define BASE64_ALPHABET "ABCDEFGHIJKLMNOPQRSTUVWXYZ" \
 			"abcdefghijklmnopqrstuvwxyz" \
-			"/+="
+			"0123456789" \
+			"+/="
 
 static char *
 my_stpcpy(char *dest, char *src)
@@ -86,6 +86,7 @@ cm_submit_u_from_file(const char *filename)
 				if (fp != stdin) {
 					fclose(fp);
 				}
+				free(csr);
 				return NULL;
 			}
 			memcpy(my_stpcpy(p, csr), buf, sizeof(buf));
@@ -115,6 +116,9 @@ cm_submit_u_from_file_single(const char *filename)
 		"-----END NEW CERTIFICATE REQUEST-----",
 	};
 	csr = cm_submit_u_from_file(filename);
+	if (csr == NULL) {
+		return NULL;
+	}
 	p = csr;
 	for (i = 0; i < sizeof(strip) / sizeof(strip[0]); i++) {
 		while ((p = strstr(csr, strip[i])) != NULL) {
@@ -187,6 +191,9 @@ cm_submit_u_pem_from_base64(const char *what, int dos, const char *base64)
 	int i;
 
 	tmp = strdup(base64);
+	if (tmp == NULL) {
+		return NULL;
+	}
 	for (p = tmp, q = base64; *q != '\0'; q++) {
 		if (strchr(BASE64_ALPHABET, *q)) {
 			*p++ = *q;
@@ -218,6 +225,7 @@ cm_submit_u_pem_from_base64(const char *what, int dos, const char *base64)
 		p = stpcpy(p, what);
 		strcpy(p, dos ? "-----\r\n" : "-----\n");
 	}
+	free(tmp);
 	return ret;
 }
 
@@ -241,6 +249,37 @@ cm_submit_princ_realm_len(krb5_context ctx, krb5_principal princ)
 #endif
 }
 
+char *
+cm_submit_u_url_encode(const char *plain)
+{
+	const char *hexchars = "0123456789ABCDEF";
+	const char *unreserved = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+				 "abcdefghijklmnopqrstuvwxyz"
+				 "0123456789-_.~";
+	char *ret = malloc(strlen(plain) * 3 + 1);
+	int i, j;
+	unsigned int c;
+
+	if (ret != NULL) {
+		for (i = 0, j = 0; plain[i] != '\0'; i++) {
+			c = ((unsigned char) plain[i]) & 0xff;
+			if (strchr(unreserved, c) != NULL) {
+				ret[j++] = plain[i];
+			} else {
+				if (c == 32) {
+					ret[j++] = '+';
+				} else {
+					ret[j++] = '%';
+					ret[j++] = hexchars[(c & 0xf0) >> 4];
+					ret[j++] = hexchars[(c & 0x0f)];
+				}
+			}
+		}
+		ret[j] = '\0';
+	}
+	return ret;
+}
+
 #ifdef HAVE_UUID
 int cm_submit_uuid_fixed_for_testing = 0;
 int
@@ -268,3 +307,102 @@ cm_submit_uuid_new(unsigned char uuid[16])
 	return 0;
 }
 #endif
+
+/* Convert a delta string to a time_t. */
+int
+cm_submit_u_delta_from_string(const char *deltas, time_t now, time_t *delta)
+{
+	struct tm now_tm, *pnow;
+	time_t start;
+	int multiple, i, val, done, digits;
+	unsigned char c;
+	val = 0;
+	digits = 0;
+	done = 0;
+	if (strlen(deltas) == 0) {
+		return -1;
+	}
+	start = now;
+	for (i = 0; !done; i++) {
+		c = (unsigned char) deltas[i];
+		switch (c) {
+		case '\0':
+			done++;
+			/* fall through */
+		case 's':
+			multiple = 1;
+			now += val * multiple;
+			val = 0;
+			break;
+		case 'm':
+			multiple = 60;
+			now += val * multiple;
+			val = 0;
+			break;
+		case 'h':
+			multiple = 60 * 60;
+			now += val * multiple;
+			val = 0;
+			break;
+		case 'd':
+			multiple = 60 * 60 * 24;
+			now += val * multiple;
+			val = 0;
+			break;
+		case 'w':
+			multiple = 60 * 60 * 24 * 7;
+			now += val * multiple;
+			val = 0;
+			break;
+		case 'M':
+			pnow = localtime_r(&now, &now_tm);
+			if (pnow == NULL) {
+				multiple = 60 * 60 * 24 * 30;
+				now += val * multiple;
+			} else {
+				now_tm.tm_mon += val;
+				now_tm.tm_year += (now_tm.tm_mon / 12);
+				now_tm.tm_mon %= 12;
+				now_tm.tm_isdst = -1; /* don't tell libc that
+						       * we "know" what's up
+						       * with DST for the time
+						       * in this structure */
+				now = mktime(&now_tm);
+			}
+			val = 0;
+			break;
+		case 'y':
+			pnow = localtime_r(&now, &now_tm);
+			if (pnow == NULL) {
+				multiple = 60 * 60 * 24 * 365;
+				now += val * multiple;
+			} else {
+				now_tm.tm_year += val;
+				now = mktime(&now_tm);
+			}
+			val = 0;
+			break;
+		case '0':
+		case '1':
+		case '2':
+		case '3':
+		case '4':
+		case '5':
+		case '6':
+		case '7':
+		case '8':
+		case '9':
+			val = (val * 10) + (c - '0');
+			digits++;
+			break;
+		default:
+			/* just skip this character */
+			break;
+		}
+	}
+	if (digits == 0) {
+		return -1;
+	}
+	*delta = now + val - start;
+	return 0;
+}
