@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009,2010,2012 Red Hat, Inc.
+ * Copyright (C) 2009,2010,2012,2013,2014 Red Hat, Inc.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -165,7 +165,8 @@ struct cm_submit_x_context {
 	xmlrpc_client *client;
 	const char *method;
 	xmlrpc_value *params, *namedarg, *results;
-	int fault_occurred:1, fault_code;
+	unsigned int fault_occurred:1;
+	int fault_code;
 	const char *fault_text;
 };
 
@@ -695,7 +696,7 @@ main(int argc, char **argv)
 				strchr(argv[0], '/') ?
 				strrchr(argv[0], '/') + 1 :
 				argv[0]);
-			return CM_STATUS_UNCONFIGURED;
+			return CM_SUBMIT_STATUS_UNCONFIGURED;
 			break;
 		}
 	}
@@ -711,15 +712,20 @@ main(int argc, char **argv)
 			strchr(argv[0], '/') ?
 			strrchr(argv[0], '/') + 1 :
 			argv[0]);
-		return CM_STATUS_UNCONFIGURED;
+		return CM_SUBMIT_STATUS_UNCONFIGURED;
 	}
-	ret = CM_STATUS_UNREACHABLE;
+	ret = CM_SUBMIT_STATUS_UNREACHABLE;
 
 	/* Read the CSR from the environment, or from the command-line. */
 	csr = getenv(CM_SUBMIT_CSR_ENV);
 	if (csr == NULL) {
 		csr = cm_submit_u_from_file((optind < argc) ?
 					    argv[optind++] : NULL);
+		if (csr == NULL) {
+			fprintf(stderr,
+				"Error reading certificate signing request.\n");
+			return CM_SUBMIT_STATUS_UNCONFIGURED;
+		}
 	}
 
 	/* Clean up the CSR. */
@@ -765,7 +771,7 @@ main(int argc, char **argv)
 			       cm_submit_x_delegate_off);
 	if (ctx == NULL) {
 		fprintf(stderr, "Error setting up for XMLRPC.\n");
-		return CM_STATUS_UNCONFIGURED;
+		return CM_SUBMIT_STATUS_UNCONFIGURED;
 	}
 
 	/* Both servers take the CSR, in their preferred format, first. */
