@@ -40,6 +40,10 @@ int cm_submit_ready(struct cm_store_entry *entry,
 int cm_submit_save_ca_cookie(struct cm_store_entry *entry,
 			     struct cm_submit_state *state);
 
+/* Clear CA-specific identifier for our submitted request. */
+int cm_submit_clear_ca_cookie(struct cm_store_entry *entry,
+			      struct cm_submit_state *state);
+
 /* If we need to poll again, any non-negative value is the polling interval. */
 int cm_submit_specified_delay(struct cm_store_entry *entry,
 			      struct cm_submit_state *state);
@@ -64,7 +68,4 @@ int cm_submit_unconfigured(struct cm_store_entry *entry,
 void cm_submit_done(struct cm_store_entry *entry,
 		    struct cm_submit_state *state);
 
-/* Convert a delta in string form to a time_t. */
-int cm_submit_delta_from_string(const char *deltas, time_t now, time_t *delta);
-
 #endif
