@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009,2011 Red Hat, Inc.
+ * Copyright (C) 2009,2011,2013 Red Hat, Inc.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,9 +21,11 @@
 #include <sys/wait.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <paths.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <syslog.h>
 #include <unistd.h>
 
 #include <talloc.h>
@@ -51,6 +53,8 @@ cm_subproc_start(int (*cb)(int fd,
 	struct cm_subproc_state *state;
 	int fds[2];
 	long flags;
+	char *configdir, *tmpdir, *tmp;
+
 	state = talloc_ptrtype(entry, state);
 	if (state != NULL) {
 		memset(state, 0, sizeof(*state));
@@ -58,23 +62,52 @@ cm_subproc_start(int (*cb)(int fd,
 		state->msg = NULL;
 		state->status = -1;
 		if (pipe(fds) != -1) {
+			fflush(NULL);
 			state->pid = fork();
 			switch (state->pid) {
 			case -1:
+				syslog(LOG_DEBUG, "fork() error: %s",
+				       strerror(errno));
 				close(fds[0]);
 				close(fds[1]);
 				talloc_free(state);
 				state = NULL;
 				break;
 			case 0:
+				state->fd = fds[1];
 				close(fds[0]);
+
+				tmp = getenv(CM_STORE_CONFIG_DIRECTORY_ENV);
+				configdir = (tmp != NULL) ? strdup(tmp) : NULL;
+				tmp = getenv("TMPDIR");
+				tmpdir = (tmp != NULL) ? strdup(tmp) : NULL;
+				clearenv();
+				setenv("HOME", CM_HOMEDIR, 1);
+				setenv("PATH", _PATH_STDPATH, 1);
+				setenv("SHELL", _PATH_BSHELL, 1);
+				setenv("TERM", "dumb", 1);
+				if (configdir != NULL) {
+					setenv(CM_STORE_CONFIG_DIRECTORY_ENV,
+					       configdir, 1);
+				}
+				if (tmpdir != NULL) {
+					setenv("TMPDIR", tmpdir, 1);
+				}
+
 				exit((*cb)(fds[1], ca, entry, data));
 				break;
 			default:
 				state->fd = fds[0];
 				flags = fcntl(state->fd, F_GETFL);
-				fcntl(state->fd, F_SETFL, flags | O_NONBLOCK);
+				if (fcntl(state->fd, F_SETFL,
+					  flags | O_NONBLOCK) != 0) {
+					syslog(LOG_DEBUG,
+					       "error marking output for "
+					       "subprocess non-blocking: %s",
+					       strerror(errno));
+				}
 				close(fds[1]);
+				fds[1] = -1;
 				break;
 			}
 		}
@@ -309,7 +342,8 @@ cm_subproc_parse_args(void *parent, const char *cmdline, const char **error)
 	return argv;
 }
 
-/* Check if we're done (return 0), or need to be called again (-1). */
+/* Redirect stdio to /dev/null, and mark everything else as close-on-exec,
+ * except for perhaps one of them that is passed in by number. */
 void
 cm_subproc_mark_most_cloexec(struct cm_store_entry *entry, int fd)
 {
@@ -354,7 +388,9 @@ cm_subproc_mark_most_cloexec(struct cm_store_entry *entry, int fd)
 		}
 		l = fcntl(i, F_GETFD);
 		if (l != -1) {
-			fcntl(i, F_SETFD, l | FD_CLOEXEC);
+			if (fcntl(i, F_SETFD, l | FD_CLOEXEC) != 0) {
+				cm_log(0, "Potentially leaking FD %d.\n", i);
+			}
 		}
 	}
 }
