@@ -47,6 +47,7 @@
 #define CM_DBUS_PROP_CERT_SERIAL "serial"
 #define CM_DBUS_PROP_CERT_SUBJECT "subject"
 #define CM_DBUS_PROP_CERT_EMAIL "email"
+#define CM_DBUS_PROP_CERT_KU "ku"
 #define CM_DBUS_PROP_CERT_EKU "eku"
 #define CM_DBUS_PROP_CERT_HOSTNAME "hostname"
 #define CM_DBUS_PROP_CERT_PRINCIPAL "principal"
@@ -59,6 +60,7 @@
 #define CM_DBUS_PROP_CSR "csr"
 #define CM_DBUS_PROP_TEMPLATE_SUBJECT "template-subject"
 #define CM_DBUS_PROP_TEMPLATE_EMAIL "template-email"
+#define CM_DBUS_PROP_TEMPLATE_KU "template-ku"
 #define CM_DBUS_PROP_TEMPLATE_EKU "template-eku"
 #define CM_DBUS_PROP_TEMPLATE_HOSTNAME "template-hostname"
 #define CM_DBUS_PROP_TEMPLATE_PRINCIPAL "template-principal"
@@ -79,11 +81,18 @@
 #define CM_DBUS_PROP_STATUS "status"
 #define CM_DBUS_PROP_STUCK "stuck"
 #define CM_DBUS_PROP_CA "ca"
+#define CM_DBUS_PROP_CA_PROFILE "ca-profile"
 #define CM_DBUS_PROP_CA_COOKIE "ca-cookie"
 #define CM_DBUS_PROP_CA_ERROR "ca-error"
 #define CM_DBUS_PROP_SUBMITTED_DATE "submitted-date"
 #define CM_DBUS_PROP_IS_DEFAULT "is-default"
 #define CM_DBUS_PROP_ISSUER_NAMES "issuer-names"
+#define CM_DBUS_PROP_TEMPLATE_IS_CA "template-is-ca"
+#define CM_DBUS_PROP_TEMPLATE_CA_PATH_LENGTH "template-ca-path-length"
+#define CM_DBUS_PROP_TEMPLATE_OCSP "template-ocsp"
+#define CM_DBUS_PROP_TEMPLATE_CRL_DP "template-crldp"
+#define CM_DBUS_PROP_TEMPLATE_NS_COMMENT "template-ns-comment"
+#define CM_DBUS_PROP_TEMPLATE_PROFILE "template-profile"
 #define CM_DBUS_SIGNAL_REQUEST_CERT_SAVED "SavedCertificate"
 
 enum cm_tdbus_type { cm_tdbus_system, cm_tdbus_session };
