@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009 Red Hat, Inc.
+ * Copyright (C) 2009,2010,2011,2012 Red Hat, Inc.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
 #include "config.h"
 
 #include <sys/types.h>
+#include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -73,12 +74,16 @@ int
 cm_tdbusm_get_b(DBusMessage *msg, void *parent, dbus_bool_t *b)
 {
 	DBusError err;
-	memset(&err, 0, sizeof(err));
+	dbus_error_init(&err);
 	if (dbus_message_get_args(msg, &err,
 				  DBUS_TYPE_BOOLEAN, b,
 				  DBUS_TYPE_INVALID)) {
 		return 0;
 	} else {
+		if (dbus_error_is_set(&err)) {
+			dbus_error_free(&err);
+			dbus_error_init(&err);
+		}
 		return -1;
 	}
 }
@@ -89,29 +94,51 @@ cm_tdbusm_get_n(DBusMessage *msg, void *parent, long *n)
 	DBusError err;
 	int64_t i64;
 	int32_t i32;
+	uint32_t u32;
 	int16_t i16;
-	memset(&err, 0, sizeof(err));
+	dbus_error_init(&err);
 	if (dbus_message_get_args(msg, &err,
 				  DBUS_TYPE_INT64, &i64,
 				  DBUS_TYPE_INVALID)) {
 		*n = i64;
 		return 0;
 	} else {
-		memset(&err, 0, sizeof(err));
+		if (dbus_error_is_set(&err)) {
+			dbus_error_free(&err);
+			dbus_error_init(&err);
+		}
 		if (dbus_message_get_args(msg, &err,
 					  DBUS_TYPE_INT32, &i32,
 					  DBUS_TYPE_INVALID)) {
 			*n = i32;
 			return 0;
 		} else {
-			memset(&err, 0, sizeof(err));
+			if (dbus_error_is_set(&err)) {
+				dbus_error_free(&err);
+				dbus_error_init(&err);
+			}
 			if (dbus_message_get_args(msg, &err,
-						  DBUS_TYPE_INT16, &i16,
+						  DBUS_TYPE_UINT32, &u32,
 						  DBUS_TYPE_INVALID)) {
-				*n = i16;
+				*n = u32;
 				return 0;
 			} else {
-				return -1;
+				if (dbus_error_is_set(&err)) {
+					dbus_error_free(&err);
+					dbus_error_init(&err);
+				}
+				if (dbus_message_get_args(msg, &err,
+							  DBUS_TYPE_INT16, &i16,
+							  DBUS_TYPE_INVALID)) {
+					*n = i16;
+					return 0;
+				} else {
+					if (dbus_error_is_set(&err)) {
+						dbus_error_free(&err);
+						dbus_error_init(&err);
+					}
+					return -1;
+				}
 			}
 		}
 	}
@@ -122,13 +149,17 @@ cm_tdbusm_get_p(DBusMessage *msg, void *parent, char **p)
 {
 	DBusError err;
 	*p = NULL;
-	memset(&err, 0, sizeof(err));
+	dbus_error_init(&err);
 	if (dbus_message_get_args(msg, &err,
 				  DBUS_TYPE_OBJECT_PATH, p,
 				  DBUS_TYPE_INVALID)) {
 		*p = *p ? talloc_strdup(parent, *p) : NULL;
 		return 0;
 	} else {
+		if (dbus_error_is_set(&err)) {
+			dbus_error_free(&err);
+			dbus_error_init(&err);
+		}
 		return -1;
 	}
 }
@@ -138,13 +169,17 @@ cm_tdbusm_get_s(DBusMessage *msg, void *parent, char **s)
 {
 	DBusError err;
 	*s = NULL;
-	memset(&err, 0, sizeof(err));
+	dbus_error_init(&err);
 	if (dbus_message_get_args(msg, &err,
 				  DBUS_TYPE_STRING, s,
 				  DBUS_TYPE_INVALID)) {
 		*s = *s ? talloc_strdup(parent, *s) : NULL;
 		return 0;
 	} else {
+		if (dbus_error_is_set(&err)) {
+			dbus_error_free(&err);
+			dbus_error_init(&err);
+		}
 		return -1;
 	}
 }
@@ -154,7 +189,7 @@ cm_tdbusm_get_bp(DBusMessage *msg, void *parent, dbus_bool_t *b, char **p)
 {
 	DBusError err;
 	*p = NULL;
-	memset(&err, 0, sizeof(err));
+	dbus_error_init(&err);
 	if (dbus_message_get_args(msg, &err,
 				  DBUS_TYPE_BOOLEAN, b,
 				  DBUS_TYPE_OBJECT_PATH, p,
@@ -162,6 +197,10 @@ cm_tdbusm_get_bp(DBusMessage *msg, void *parent, dbus_bool_t *b, char **p)
 		*p = *p ? talloc_strdup(parent, *p) : NULL;
 		return 0;
 	} else {
+		if (dbus_error_is_set(&err)) {
+			dbus_error_free(&err);
+			dbus_error_init(&err);
+		}
 		return -1;
 	}
 }
@@ -171,7 +210,7 @@ cm_tdbusm_get_bs(DBusMessage *msg, void *parent, dbus_bool_t *b, char **s)
 {
 	DBusError err;
 	*s = NULL;
-	memset(&err, 0, sizeof(err));
+	dbus_error_init(&err);
 	if (dbus_message_get_args(msg, &err,
 				  DBUS_TYPE_BOOLEAN, b,
 				  DBUS_TYPE_STRING, s,
@@ -179,6 +218,10 @@ cm_tdbusm_get_bs(DBusMessage *msg, void *parent, dbus_bool_t *b, char **s)
 		*s = *s ? talloc_strdup(parent, *s) : NULL;
 		return 0;
 	} else {
+		if (dbus_error_is_set(&err)) {
+			dbus_error_free(&err);
+			dbus_error_init(&err);
+		}
 		return -1;
 	}
 }
@@ -188,7 +231,7 @@ cm_tdbusm_get_sb(DBusMessage *msg, void *parent, char **s, dbus_bool_t *b)
 {
 	DBusError err;
 	*s = NULL;
-	memset(&err, 0, sizeof(err));
+	dbus_error_init(&err);
 	if (dbus_message_get_args(msg, &err,
 				  DBUS_TYPE_STRING, s,
 				  DBUS_TYPE_BOOLEAN, b,
@@ -196,6 +239,10 @@ cm_tdbusm_get_sb(DBusMessage *msg, void *parent, char **s, dbus_bool_t *b)
 		*s = *s ? talloc_strdup(parent, *s) : NULL;
 		return 0;
 	} else {
+		if (dbus_error_is_set(&err)) {
+			dbus_error_free(&err);
+			dbus_error_init(&err);
+		}
 		return -1;
 	}
 }
@@ -208,7 +255,7 @@ cm_tdbusm_get_sn(DBusMessage *msg, void *parent, char **s, long *n)
 	int64_t i32;
 	int64_t i16;
 	*s = NULL;
-	memset(&err, 0, sizeof(err));
+	dbus_error_init(&err);
 	if (dbus_message_get_args(msg, &err,
 				  DBUS_TYPE_STRING, s,
 				  DBUS_TYPE_INT64, &i64,
@@ -217,7 +264,10 @@ cm_tdbusm_get_sn(DBusMessage *msg, void *parent, char **s, long *n)
 		*n = i64;
 		return 0;
 	} else {
-		memset(&err, 0, sizeof(err));
+		if (dbus_error_is_set(&err)) {
+			dbus_error_free(&err);
+			dbus_error_init(&err);
+		}
 		if (dbus_message_get_args(msg, &err,
 					  DBUS_TYPE_STRING, s,
 					  DBUS_TYPE_INT32, &i32,
@@ -226,7 +276,10 @@ cm_tdbusm_get_sn(DBusMessage *msg, void *parent, char **s, long *n)
 			*n = i32;
 			return 0;
 		} else {
-			memset(&err, 0, sizeof(err));
+			if (dbus_error_is_set(&err)) {
+				dbus_error_free(&err);
+				dbus_error_init(&err);
+			}
 			if (dbus_message_get_args(msg, &err,
 						  DBUS_TYPE_STRING, s,
 						  DBUS_TYPE_INT16, &i16,
@@ -235,6 +288,10 @@ cm_tdbusm_get_sn(DBusMessage *msg, void *parent, char **s, long *n)
 				*n = i16;
 				return 0;
 			} else {
+				if (dbus_error_is_set(&err)) {
+					dbus_error_free(&err);
+					dbus_error_init(&err);
+				}
 				return -1;
 			}
 		}
@@ -247,7 +304,7 @@ cm_tdbusm_get_ss(DBusMessage *msg, void *parent, char **s1, char **s2)
 	DBusError err;
 	*s1 = NULL;
 	*s2 = NULL;
-	memset(&err, 0, sizeof(err));
+	dbus_error_init(&err);
 	if (dbus_message_get_args(msg, &err,
 				  DBUS_TYPE_STRING, s1,
 				  DBUS_TYPE_STRING, s2,
@@ -256,6 +313,10 @@ cm_tdbusm_get_ss(DBusMessage *msg, void *parent, char **s1, char **s2)
 		*s2 = *s2 ? talloc_strdup(parent, *s2) : NULL;
 		return 0;
 	} else {
+		if (dbus_error_is_set(&err)) {
+			dbus_error_free(&err);
+			dbus_error_init(&err);
+		}
 		return -1;
 	}
 }
@@ -267,7 +328,7 @@ cm_tdbusm_get_ap(DBusMessage *msg, void *parent, char ***ap)
 	char **tmp;
 	int i;
 	*ap = NULL;
-	memset(&err, 0, sizeof(err));
+	dbus_error_init(&err);
 	if (dbus_message_get_args(msg, &err,
 				  DBUS_TYPE_ARRAY, DBUS_TYPE_OBJECT_PATH,
 				  &tmp, &i,
@@ -275,6 +336,10 @@ cm_tdbusm_get_ap(DBusMessage *msg, void *parent, char ***ap)
 		*ap = cm_tdbusm_take_dbus_string_array(parent, tmp, i);
 		return 0;
 	} else {
+		if (dbus_error_is_set(&err)) {
+			dbus_error_free(&err);
+			dbus_error_init(&err);
+		}
 		return -1;
 	}
 }
@@ -286,13 +351,17 @@ cm_tdbusm_get_as(DBusMessage *msg, void *parent, char ***as)
 	char **tmp;
 	int i;
 	*as = NULL;
-	memset(&err, 0, sizeof(err));
+	dbus_error_init(&err);
 	if (dbus_message_get_args(msg, &err,
 				  DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, &tmp, &i,
 				  DBUS_TYPE_INVALID)) {
 		*as = cm_tdbusm_take_dbus_string_array(parent, tmp, i);
 		return 0;
 	} else {
+		if (dbus_error_is_set(&err)) {
+			dbus_error_free(&err);
+			dbus_error_init(&err);
+		}
 		return -1;
 	}
 }
@@ -305,7 +374,7 @@ cm_tdbusm_get_sss(DBusMessage *msg, void *parent, char **s1, char **s2,
 	*s1 = NULL;
 	*s2 = NULL;
 	*s3 = NULL;
-	memset(&err, 0, sizeof(err));
+	dbus_error_init(&err);
 	if (dbus_message_get_args(msg, &err,
 				  DBUS_TYPE_STRING, s1,
 				  DBUS_TYPE_STRING, s2,
@@ -316,10 +385,89 @@ cm_tdbusm_get_sss(DBusMessage *msg, void *parent, char **s1, char **s2,
 		*s3 = *s3 ? talloc_strdup(parent, *s3) : NULL;
 		return 0;
 	} else {
+		if (dbus_error_is_set(&err)) {
+			dbus_error_free(&err);
+			dbus_error_init(&err);
+		}
 		return -1;
 	}
 }
 
+int
+cm_tdbusm_get_ssb(DBusMessage *msg, void *parent, char **s1, char **s2,
+		  dbus_bool_t *b)
+{
+	DBusError err;
+	*s1 = NULL;
+	*s2 = NULL;
+	dbus_error_init(&err);
+	if (dbus_message_get_args(msg, &err,
+				  DBUS_TYPE_STRING, s1,
+				  DBUS_TYPE_STRING, s2,
+				  DBUS_TYPE_BOOLEAN, b,
+				  DBUS_TYPE_INVALID)) {
+		*s1 = *s1 ? talloc_strdup(parent, *s1) : NULL;
+		*s2 = *s2 ? talloc_strdup(parent, *s2) : NULL;
+		return 0;
+	} else {
+		if (dbus_error_is_set(&err)) {
+			dbus_error_free(&err);
+			dbus_error_init(&err);
+		}
+		return -1;
+	}
+}
+
+int
+cm_tdbusm_get_ssn(DBusMessage *msg, void *parent, char **s1, char **s2, long *l)
+{
+	DBusError err;
+	int64_t i64;
+	int32_t i32;
+	int16_t i16;
+	*s1 = NULL;
+	*s2 = NULL;
+
+	dbus_error_init(&err);
+	if (!dbus_message_get_args(msg, &err,
+				   DBUS_TYPE_STRING, s1,
+				   DBUS_TYPE_STRING, s2,
+				   DBUS_TYPE_INT64, &i64,
+				   DBUS_TYPE_INVALID)) {
+		if (dbus_error_is_set(&err)) {
+			dbus_error_free(&err);
+			dbus_error_init(&err);
+		}
+		if (!dbus_message_get_args(msg, &err,
+					   DBUS_TYPE_STRING, s1,
+					   DBUS_TYPE_STRING, s2,
+					   DBUS_TYPE_INT32, &i32,
+					   DBUS_TYPE_INVALID)) {
+			if (dbus_error_is_set(&err)) {
+				dbus_error_free(&err);
+				dbus_error_init(&err);
+			}
+			if (!dbus_message_get_args(msg, &err,
+						   DBUS_TYPE_STRING, s1,
+						   DBUS_TYPE_STRING, s2,
+						   DBUS_TYPE_INT16, &i16,
+						   DBUS_TYPE_INVALID)) {
+				if (dbus_error_is_set(&err)) {
+					dbus_error_free(&err);
+					dbus_error_init(&err);
+				}
+				return -1;
+			}
+			i32 = i16;
+		}
+		i64 = i32;
+	}
+	*l = i64;
+	*s1 = *s1 ? talloc_strdup(parent, *s1) : NULL;
+	*s2 = *s2 ? talloc_strdup(parent, *s2) : NULL;
+	return 0;
+}
+
 int
 cm_tdbusm_get_ssss(DBusMessage *msg, void *parent, char **s1, char **s2,
 		   char **s3, char **s4)
@@ -329,7 +477,7 @@ cm_tdbusm_get_ssss(DBusMessage *msg, void *parent, char **s1, char **s2,
 	*s2 = NULL;
 	*s3 = NULL;
 	*s4 = NULL;
-	memset(&err, 0, sizeof(err));
+	dbus_error_init(&err);
 	if (dbus_message_get_args(msg, &err,
 				  DBUS_TYPE_STRING, s1,
 				  DBUS_TYPE_STRING, s2,
@@ -342,6 +490,10 @@ cm_tdbusm_get_ssss(DBusMessage *msg, void *parent, char **s1, char **s2,
 		*s4 = *s4 ? talloc_strdup(parent, *s4) : NULL;
 		return 0;
 	} else {
+		if (dbus_error_is_set(&err)) {
+			dbus_error_free(&err);
+			dbus_error_init(&err);
+		}
 		return -1;
 	}
 }
@@ -394,7 +546,7 @@ cm_tdbusm_get_ssas(DBusMessage *msg, void *parent,
 	*s1 = NULL;
 	*s2 = NULL;
 	*as = NULL;
-	memset(&err, 0, sizeof(err));
+	dbus_error_init(&err);
 	if (dbus_message_get_args(msg, &err,
 				  DBUS_TYPE_STRING, s1,
 				  DBUS_TYPE_STRING, s2,
@@ -405,6 +557,10 @@ cm_tdbusm_get_ssas(DBusMessage *msg, void *parent,
 		*as = cm_tdbusm_take_dbus_string_array(parent, tmp, i);
 		return 0;
 	} else {
+		if (dbus_error_is_set(&err)) {
+			dbus_error_free(&err);
+			dbus_error_init(&err);
+		}
 		return -1;
 	}
 }
@@ -420,7 +576,7 @@ cm_tdbusm_get_sssas(DBusMessage *msg, void *parent,
 	*s2 = NULL;
 	*s3 = NULL;
 	*as = NULL;
-	memset(&err, 0, sizeof(err));
+	dbus_error_init(&err);
 	if (dbus_message_get_args(msg, &err,
 				  DBUS_TYPE_STRING, s1,
 				  DBUS_TYPE_STRING, s2,
@@ -433,6 +589,10 @@ cm_tdbusm_get_sssas(DBusMessage *msg, void *parent,
 		*as = cm_tdbusm_take_dbus_string_array(parent, tmp, i);
 		return 0;
 	} else {
+		if (dbus_error_is_set(&err)) {
+			dbus_error_free(&err);
+			dbus_error_init(&err);
+		}
 		return -1;
 	}
 }
@@ -473,7 +633,7 @@ cm_tdbusm_get_sssnasasasnas(DBusMessage *msg, void *parent,
 	*as2 = NULL;
 	*as3 = NULL;
 	*as4 = NULL;
-	memset(&err, 0, sizeof(err));
+	dbus_error_init(&err);
 	if (!dbus_message_get_args(msg, &err,
 				   DBUS_TYPE_STRING, s1,
 				   DBUS_TYPE_STRING, s2,
@@ -485,6 +645,10 @@ cm_tdbusm_get_sssnasasasnas(DBusMessage *msg, void *parent,
 				   DBUS_TYPE_INT64, &i642,
 				   DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, &tmp4, &l,
 				   DBUS_TYPE_INVALID)) {
+		if (dbus_error_is_set(&err)) {
+			dbus_error_free(&err);
+			dbus_error_init(&err);
+		}
 		if (!dbus_message_get_args(msg, &err,
 					   DBUS_TYPE_STRING, s1,
 					   DBUS_TYPE_STRING, s2,
@@ -500,6 +664,10 @@ cm_tdbusm_get_sssnasasasnas(DBusMessage *msg, void *parent,
 					   DBUS_TYPE_ARRAY, DBUS_TYPE_STRING,
 					   &tmp4, &l,
 					   DBUS_TYPE_INVALID)) {
+			if (dbus_error_is_set(&err)) {
+				dbus_error_free(&err);
+				dbus_error_init(&err);
+			}
 			if (!dbus_message_get_args(msg, &err,
 						   DBUS_TYPE_STRING, s1,
 						   DBUS_TYPE_STRING, s2,
@@ -515,6 +683,10 @@ cm_tdbusm_get_sssnasasasnas(DBusMessage *msg, void *parent,
 						   DBUS_TYPE_ARRAY,
 						   DBUS_TYPE_STRING, &tmp4, &l,
 						   DBUS_TYPE_INVALID)) {
+				if (dbus_error_is_set(&err)) {
+					dbus_error_free(&err);
+					dbus_error_init(&err);
+				}
 				return -1;
 			}
 			i321 = i161;
@@ -551,7 +723,7 @@ cm_tdbusm_get_sasasasnas(DBusMessage *msg, void *parent, char **s,
 	*as2 = NULL;
 	*as3 = NULL;
 	*as4 = NULL;
-	memset(&err, 0, sizeof(err));
+	dbus_error_init(&err);
 	if (!dbus_message_get_args(msg, &err,
 				   DBUS_TYPE_STRING, s,
 				   DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, &tmp1, &i,
@@ -560,6 +732,10 @@ cm_tdbusm_get_sasasasnas(DBusMessage *msg, void *parent, char **s,
 				   DBUS_TYPE_INT64, &i64,
 				   DBUS_TYPE_ARRAY, DBUS_TYPE_STRING, &tmp4, &l,
 				   DBUS_TYPE_INVALID)) {
+		if (dbus_error_is_set(&err)) {
+			dbus_error_free(&err);
+			dbus_error_init(&err);
+		}
 		if (!dbus_message_get_args(msg, &err,
 					   DBUS_TYPE_STRING, s,
 					   DBUS_TYPE_ARRAY,
@@ -572,6 +748,10 @@ cm_tdbusm_get_sasasasnas(DBusMessage *msg, void *parent, char **s,
 					   DBUS_TYPE_ARRAY,
 					   DBUS_TYPE_STRING, &tmp4, &l,
 					   DBUS_TYPE_INVALID)) {
+			if (dbus_error_is_set(&err)) {
+				dbus_error_free(&err);
+				dbus_error_init(&err);
+			}
 			if (!dbus_message_get_args(msg, &err,
 						   DBUS_TYPE_STRING, s,
 						   DBUS_TYPE_ARRAY,
@@ -584,6 +764,10 @@ cm_tdbusm_get_sasasasnas(DBusMessage *msg, void *parent, char **s,
 						   DBUS_TYPE_ARRAY,
 						   DBUS_TYPE_STRING, &tmp4, &l,
 						   DBUS_TYPE_INVALID)) {
+				if (dbus_error_is_set(&err)) {
+					dbus_error_free(&err);
+					dbus_error_init(&err);
+				}
 				return -1;
 			}
 			i32 = i16;
@@ -637,6 +821,12 @@ cm_tdbusm_get_d_item(DBusMessageIter *item, void *parent)
 			dict->value_type = cm_tdbusm_dict_b;
 			dbus_message_iter_get_basic(&value, &dict->value.b);
 			break;
+		/* It can be a path. */
+		case DBUS_TYPE_OBJECT_PATH:
+			dict->value_type = cm_tdbusm_dict_p;
+			dbus_message_iter_get_basic(&value, &s);
+			dict->value.s = talloc_strdup(dict, s);
+			break;
 		/* It can be a string. */
 		case DBUS_TYPE_STRING:
 			dict->value_type = cm_tdbusm_dict_s;
@@ -816,6 +1006,68 @@ cm_tdbusm_get_d(DBusMessage *msg, void *parent, struct cm_tdbusm_dict ***d)
 	return -1;
 }
 
+int
+cm_tdbusm_get_sd(DBusMessage *msg, void *parent,
+		 char **s, struct cm_tdbusm_dict ***d)
+{
+	struct cm_tdbusm_dict **tdicts, **dicts, **tmp;
+	DBusMessageIter args, array;
+	int i, n_dicts;
+	*d = NULL;
+	dicts = NULL;
+	n_dicts = 0;
+	memset(&args, 0, sizeof(args));
+	if (dbus_message_iter_init(msg, &args)) {
+		if (dbus_message_iter_get_arg_type(&args) != DBUS_TYPE_STRING) {
+			return -1;
+		}
+		dbus_message_iter_get_basic(&args, s);
+		if (*s == NULL) {
+			return -1;
+		}
+		*s = talloc_strdup(parent, *s);
+		if (!dbus_message_iter_has_next(&args) ||
+		    !dbus_message_iter_next(&args)) {
+			return -1;
+		}
+		if (dbus_message_iter_get_arg_type(&args) != DBUS_TYPE_ARRAY) {
+			return -1;
+		}
+		memset(&array, 0, sizeof(array));
+		dbus_message_iter_recurse(&args, &array);
+		tdicts = cm_tdbusm_get_d_array(&array, parent);
+		if (tdicts != NULL) {
+			for (i = 0; tdicts[i] != NULL; i++) {
+				continue;
+			}
+			tmp = talloc_realloc(parent, dicts,
+					     struct cm_tdbusm_dict *,
+					     n_dicts + i + 1);
+			if (tmp != NULL) {
+				memcpy(tmp + n_dicts,
+				       tdicts,
+				       i * sizeof(tdicts[0]));
+				n_dicts += i;
+				tmp[n_dicts] = NULL;
+				dicts = tmp;
+			} else {
+				talloc_free(tdicts);
+				talloc_free(dicts);
+				return -1;
+			}
+		}
+		if (dbus_message_iter_has_next(&args)) {
+			if (!dbus_message_iter_next(&args)) {
+				talloc_free(dicts);
+				return -1;
+			}
+		}
+		*d = dicts;
+		return 0;
+	}
+	return -1;
+}
+
 int
 cm_tdbusm_set_b(DBusMessage *msg, dbus_bool_t b)
 {
@@ -949,6 +1201,49 @@ cm_tdbusm_set_ss(DBusMessage *msg, const char *s1, const char *s2)
 	}
 }
 
+int
+cm_tdbusm_set_ssb(DBusMessage *msg, const char *s1, const char *s2,
+		  dbus_bool_t b)
+{
+	if (s1 == NULL) {
+		s1 = empty_string;
+	}
+	if (s2 == NULL) {
+		s2 = empty_string;
+	}
+	if (dbus_message_append_args(msg,
+				     DBUS_TYPE_STRING, &s1,
+				     DBUS_TYPE_STRING, &s2,
+				     DBUS_TYPE_BOOLEAN, &b,
+				     DBUS_TYPE_INVALID)) {
+		return 0;
+	} else {
+		return -1;
+	}
+}
+
+int
+cm_tdbusm_set_ssn(DBusMessage *msg, const char *s1, const char *s2,
+		  long n)
+{
+	int64_t i = n;
+	if (s1 == NULL) {
+		s1 = empty_string;
+	}
+	if (s2 == NULL) {
+		s2 = empty_string;
+	}
+	if (dbus_message_append_args(msg,
+				     DBUS_TYPE_STRING, &s1,
+				     DBUS_TYPE_STRING, &s2,
+				     DBUS_TYPE_INT64, &i,
+				     DBUS_TYPE_INVALID)) {
+		return 0;
+	} else {
+		return -1;
+	}
+}
+
 int
 cm_tdbusm_set_ap(DBusMessage *msg, const char **ap)
 {
@@ -1030,6 +1325,32 @@ cm_tdbusm_set_ssss(DBusMessage *msg, const char *s1, const char *s2,
 	}
 }
 
+int
+cm_tdbusm_set_ssas(DBusMessage *msg,
+		   const char *s1, const char *s2, const char **as)
+{
+	if (s1 == NULL) {
+		s1 = empty_string;
+	}
+	if (s2 == NULL) {
+		s2 = empty_string;
+	}
+	if (as == NULL) {
+		as = empty_string_array;
+	}
+	if (dbus_message_append_args(msg,
+				     DBUS_TYPE_STRING, &s1,
+				     DBUS_TYPE_STRING, &s2,
+				     DBUS_TYPE_ARRAY, DBUS_TYPE_STRING,
+				     &as, cm_tdbusm_array_length(as),
+				     DBUS_TYPE_INVALID)) {
+		return 0;
+	} else {
+		return -1;
+	}
+}
+
+
 int
 cm_tdbusm_set_ssoas(DBusMessage *msg,
 		    const char *s1, const char *s2, const char **as)
@@ -1173,14 +1494,18 @@ cm_tdbusm_set_sasasasnas(DBusMessage *msg, const char *s,
 	}
 }
 
-int
-cm_tdbusm_set_d(DBusMessage *msg, const struct cm_tdbusm_dict **d)
+static int
+cm_tdbusm_set_osd(DBusMessage *msg,
+		  const char *s, const struct cm_tdbusm_dict **d)
 {
 	DBusMessageIter args, array, entry, val, elt;
 	int i;
 	int64_t l;
 	memset(&args, 0, sizeof(args));
 	dbus_message_iter_init_append(msg, &args);
+	if (s != NULL) {
+		dbus_message_iter_append_basic(&args, DBUS_TYPE_STRING, &s);
+	}
 	memset(&array, 0, sizeof(array));
 	dbus_message_iter_open_container(&args,
 					 DBUS_TYPE_ARRAY,
@@ -1219,6 +1544,16 @@ cm_tdbusm_set_d(DBusMessage *msg, const struct cm_tdbusm_dict **d)
 						       &l);
 			dbus_message_iter_close_container(&entry, &val);
 			break;
+		case cm_tdbusm_dict_p:
+			dbus_message_iter_open_container(&entry,
+							 DBUS_TYPE_VARIANT,
+							 DBUS_TYPE_OBJECT_PATH_AS_STRING,
+							 &val);
+			dbus_message_iter_append_basic(&val,
+						       DBUS_TYPE_OBJECT_PATH,
+						       &d[i]->value.s);
+			dbus_message_iter_close_container(&entry, &val);
+			break;
 		case cm_tdbusm_dict_s:
 			dbus_message_iter_open_container(&entry,
 							 DBUS_TYPE_VARIANT,
@@ -1258,6 +1593,22 @@ cm_tdbusm_set_d(DBusMessage *msg, const struct cm_tdbusm_dict **d)
 	return (i > 0) ? 0 : -1;
 }
 
+int
+cm_tdbusm_set_d(DBusMessage *msg, const struct cm_tdbusm_dict **d)
+{
+	return cm_tdbusm_set_osd(msg, NULL, d);
+}
+
+int
+cm_tdbusm_set_sd(DBusMessage *msg,
+		 const char *s, const struct cm_tdbusm_dict **d)
+{
+	if (s == NULL) {
+		return -1;
+	}
+	return cm_tdbusm_set_osd(msg, s, d);
+}
+
 struct cm_tdbusm_dict *
 cm_tdbusm_find_dict_entry(struct cm_tdbusm_dict **d,
 			  const char *key,
@@ -1271,6 +1622,16 @@ cm_tdbusm_find_dict_entry(struct cm_tdbusm_dict **d,
 		    (strcasecmp(key, d[i]->key) == 0)) {
 			ret = d[i];
 		}
+		if ((value_type == cm_tdbusm_dict_p) &&
+		    (d[i]->value_type == cm_tdbusm_dict_s) &&
+		    (strcasecmp(key, d[i]->key) == 0)) {
+			ret = d[i];
+		}
+		if ((value_type == cm_tdbusm_dict_s) &&
+		    (d[i]->value_type == cm_tdbusm_dict_p) &&
+		    (strcasecmp(key, d[i]->key) == 0)) {
+			ret = d[i];
+		}
 	}
 	return ret;
 }
@@ -1279,6 +1640,9 @@ char *
 cm_tdbusm_hint(void *parent, const char *error, const char *message)
 {
 	char *text = NULL;
+	if (error == NULL) {
+		return NULL;
+	}
 	if (strcmp(error, DBUS_ERROR_ACCESS_DENIED) == 0) {
 		text = N_("Insufficient access.  Please retry operation as root.\n");
 	} else
