@@ -16,6 +16,8 @@
  */
 
 #include "config.h"
+#include <string.h>
+#include <openssl/bn.h>
 #include <openssl/pem.h>
 #include "util-o.h"
 
@@ -30,3 +32,25 @@ util_o_init(void)
 	SSL_library_init();
 #endif
 }
+
+char *
+util_o_dec_from_hex(const char *hex)
+{
+	BIGNUM *bn = NULL;
+	char *tmp, *ret = NULL;
+
+	if (strlen(hex) > 0) {
+		if (BN_hex2bn(&bn, hex) == 0) {
+			return NULL;
+		}
+		tmp = BN_bn2dec(bn);
+		BN_free(bn);
+		if (tmp != NULL) {
+			ret = strdup(tmp);
+			OPENSSL_free(tmp);
+		}
+	} else {
+		ret = strdup("");
+	}
+	return ret;
+}
